# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/setup_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SetupDialog(object):
    def setupUi(self, SetupDialog):
        SetupDialog.setObjectName("SetupDialog")
        SetupDialog.resize(555, 400)
        self.verticalLayout = QtWidgets.QVBoxLayout(SetupDialog)
        self.verticalLayout.setContentsMargins(-1, 15, 15, 24)
        self.verticalLayout.setSpacing(5)
        self.verticalLayout.setObjectName("verticalLayout")
        self.stackedWidget = AnimatedStackedWidget(SetupDialog)
        self.stackedWidget.setObjectName("stackedWidget")
        self.welcomePage = QtWidgets.QWidget()
        self.welcomePage.setObjectName("welcomePage")
        self.gridLayout = QtWidgets.QGridLayout(self.welcomePage)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButtonLink = QtWidgets.QPushButton(self.welcomePage)
        self.pushButtonLink.setObjectName("pushButtonLink")
        self.gridLayout.addWidget(self.pushButtonLink, 4, 0, 1, 3, QtCore.Qt.AlignmentFlag.AlignHCenter)
        self.labelWelcome = QtWidgets.QLabel(self.welcomePage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelWelcome.sizePolicy().hasHeightForWidth())
        self.labelWelcome.setSizePolicy(sizePolicy)
        self.labelWelcome.setMinimumSize(QtCore.QSize(0, 50))
        self.labelWelcome.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.labelWelcome.setObjectName("labelWelcome")
        self.gridLayout.addWidget(self.labelWelcome, 3, 0, 1, 3)
        self.labelIcon_0 = QtWidgets.QLabel(self.welcomePage)
        self.labelIcon_0.setMinimumSize(QtCore.QSize(150, 150))
        self.labelIcon_0.setMaximumSize(QtCore.QSize(150, 150))
        self.labelIcon_0.setText("")
        self.labelIcon_0.setScaledContents(True)
        self.labelIcon_0.setWordWrap(False)
        self.labelIcon_0.setObjectName("labelIcon_0")
        self.gridLayout.addWidget(self.labelIcon_0, 1, 1, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem, 5, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem1, 0, 0, 1, 3)
        spacerItem2 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem2, 2, 0, 1, 3)
        self.stackedWidget.addWidget(self.welcomePage)
        self.authPage = QtWidgets.QWidget()
        self.authPage.setObjectName("authPage")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.authPage)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.progressIndicator = QProgressIndicator(self.authPage)
        self.progressIndicator.setMinimumSize(QtCore.QSize(32, 32))
        self.progressIndicator.setMaximumSize(QtCore.QSize(32, 32))
        self.progressIndicator.setObjectName("progressIndicator")
        self.gridLayout_5.addWidget(self.progressIndicator, 5, 0, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter|QtCore.Qt.AlignmentFlag.AlignVCenter)
        spacerItem3 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Preferred)
        self.gridLayout_5.addItem(spacerItem3, 2, 0, 1, 1)
        self.labelAuthLink = QtWidgets.QLabel(self.authPage)
        self.labelAuthLink.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.labelAuthLink.setWordWrap(True)
        self.labelAuthLink.setOpenExternalLinks(True)
        self.labelAuthLink.setObjectName("labelAuthLink")
        self.gridLayout_5.addWidget(self.labelAuthLink, 3, 0, 1, 1)
        self.lineEditAuthCode = QtWidgets.QLineEdit(self.authPage)
        self.lineEditAuthCode.setMinimumSize(QtCore.QSize(500, 0))
        self.lineEditAuthCode.setMaximumSize(QtCore.QSize(500, 16777215))
        self.lineEditAuthCode.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.lineEditAuthCode.setObjectName("lineEditAuthCode")
        self.gridLayout_5.addWidget(self.lineEditAuthCode, 4, 0, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        self.labelIcon_1 = QtWidgets.QLabel(self.authPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIcon_1.sizePolicy().hasHeightForWidth())
        self.labelIcon_1.setSizePolicy(sizePolicy)
        self.labelIcon_1.setMinimumSize(QtCore.QSize(70, 70))
        self.labelIcon_1.setMaximumSize(QtCore.QSize(70, 70))
        self.labelIcon_1.setText("")
        self.labelIcon_1.setObjectName("labelIcon_1")
        self.gridLayout_5.addWidget(self.labelIcon_1, 1, 0, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        spacerItem4 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.gridLayout_5.addItem(spacerItem4, 0, 0, 1, 1)
        self.stackedWidget.addWidget(self.authPage)
        self.dropboxPathPage = QtWidgets.QWidget()
        self.dropboxPathPage.setObjectName("dropboxPathPage")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.dropboxPathPage)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        spacerItem5 = QtWidgets.QSpacerItem(20, 15, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.gridLayout_7.addItem(spacerItem5, 0, 0, 1, 1)
        self.labelIcon_2 = QtWidgets.QLabel(self.dropboxPathPage)
        self.labelIcon_2.setMinimumSize(QtCore.QSize(70, 70))
        self.labelIcon_2.setMaximumSize(QtCore.QSize(70, 70))
        self.labelIcon_2.setText("")
        self.labelIcon_2.setObjectName("labelIcon_2")
        self.gridLayout_7.addWidget(self.labelIcon_2, 1, 0, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        self.labelDropboxPath = QtWidgets.QLabel(self.dropboxPathPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelDropboxPath.sizePolicy().hasHeightForWidth())
        self.labelDropboxPath.setSizePolicy(sizePolicy)
        self.labelDropboxPath.setMinimumSize(QtCore.QSize(0, 20))
        self.labelDropboxPath.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.labelDropboxPath.setWordWrap(True)
        self.labelDropboxPath.setObjectName("labelDropboxPath")
        self.gridLayout_7.addWidget(self.labelDropboxPath, 3, 0, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 15, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.gridLayout_7.addItem(spacerItem6, 2, 0, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(20, 50, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_7.addItem(spacerItem7, 5, 0, 1, 1)
        self.comboBoxDropboxPath = QtWidgets.QComboBox(self.dropboxPathPage)
        self.comboBoxDropboxPath.setObjectName("comboBoxDropboxPath")
        self.gridLayout_7.addWidget(self.comboBoxDropboxPath, 4, 0, 1, 1)
        self.stackedWidget.addWidget(self.dropboxPathPage)
        self.excludedFoldersPage = QtWidgets.QWidget()
        self.excludedFoldersPage.setObjectName("excludedFoldersPage")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.excludedFoldersPage)
        self.gridLayout_9.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.labelFolderSelection = QtWidgets.QLabel(self.excludedFoldersPage)
        self.labelFolderSelection.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelFolderSelection.setWordWrap(True)
        self.labelFolderSelection.setObjectName("labelFolderSelection")
        self.gridLayout_9.addWidget(self.labelFolderSelection, 0, 0, 1, 1)
        self.selectAllCheckBox = QtWidgets.QCheckBox(self.excludedFoldersPage)
        self.selectAllCheckBox.setEnabled(False)
        self.selectAllCheckBox.setObjectName("selectAllCheckBox")
        self.gridLayout_9.addWidget(self.selectAllCheckBox, 2, 0, 1, 1)
        self.treeViewFolders = QtWidgets.QTreeView(self.excludedFoldersPage)
        self.treeViewFolders.setEditTriggers(QtWidgets.QAbstractItemView.EditTrigger.NoEditTriggers)
        self.treeViewFolders.setAlternatingRowColors(True)
        self.treeViewFolders.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.NoSelection)
        self.treeViewFolders.setUniformRowHeights(True)
        self.treeViewFolders.setSortingEnabled(True)
        self.treeViewFolders.setHeaderHidden(True)
        self.treeViewFolders.setObjectName("treeViewFolders")
        self.treeViewFolders.header().setVisible(False)
        self.treeViewFolders.header().setMinimumSectionSize(50)
        self.gridLayout_9.addWidget(self.treeViewFolders, 1, 0, 1, 1)
        self.stackedWidget.addWidget(self.excludedFoldersPage)
        self.donePage = QtWidgets.QWidget()
        self.donePage.setObjectName("donePage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.donePage)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem8 = QtWidgets.QSpacerItem(20, 30, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Preferred)
        self.gridLayout_3.addItem(spacerItem8, 5, 0, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem9, 2, 0, 1, 1)
        self.labelIcon_3 = QtWidgets.QLabel(self.donePage)
        self.labelIcon_3.setMinimumSize(QtCore.QSize(120, 120))
        self.labelIcon_3.setMaximumSize(QtCore.QSize(120, 120))
        self.labelIcon_3.setText("")
        self.labelIcon_3.setObjectName("labelIcon_3")
        self.gridLayout_3.addWidget(self.labelIcon_3, 1, 0, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        self.labelSetupComplete = QtWidgets.QLabel(self.donePage)
        self.labelSetupComplete.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.labelSetupComplete.setWordWrap(True)
        self.labelSetupComplete.setObjectName("labelSetupComplete")
        self.gridLayout_3.addWidget(self.labelSetupComplete, 3, 0, 1, 1)
        self.pushButtonClose = QtWidgets.QPushButton(self.donePage)
        icon = QtGui.QIcon.fromTheme("dialog-apply")
        self.pushButtonClose.setIcon(icon)
        self.pushButtonClose.setObjectName("pushButtonClose")
        self.gridLayout_3.addWidget(self.pushButtonClose, 4, 0, 1, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        spacerItem10 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem10, 0, 0, 1, 1)
        self.stackedWidget.addWidget(self.donePage)
        self.verticalLayout.addWidget(self.stackedWidget)
        self.stackedWidgetButtons = QtWidgets.QStackedWidget(SetupDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.stackedWidgetButtons.sizePolicy().hasHeightForWidth())
        self.stackedWidgetButtons.setSizePolicy(sizePolicy)
        self.stackedWidgetButtons.setObjectName("stackedWidgetButtons")
        self.welcomePageButtons = QtWidgets.QWidget()
        self.welcomePageButtons.setObjectName("welcomePageButtons")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.welcomePageButtons)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem11 = QtWidgets.QSpacerItem(50, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem11, 0, 0, 1, 1)
        self.stackedWidgetButtons.addWidget(self.welcomePageButtons)
        self.authPageButtons = QtWidgets.QWidget()
        self.authPageButtons.setObjectName("authPageButtons")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.authPageButtons)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.pushButtonAuthPageLink = QtWidgets.QPushButton(self.authPageButtons)
        self.pushButtonAuthPageLink.setEnabled(True)
        icon = QtGui.QIcon.fromTheme("dialog-ok")
        self.pushButtonAuthPageLink.setIcon(icon)
        self.pushButtonAuthPageLink.setObjectName("pushButtonAuthPageLink")
        self.gridLayout_4.addWidget(self.pushButtonAuthPageLink, 0, 2, 1, 1)
        self.pushButtonAuthPageCancel = QtWidgets.QPushButton(self.authPageButtons)
        icon = QtGui.QIcon.fromTheme("dialog-close")
        self.pushButtonAuthPageCancel.setIcon(icon)
        self.pushButtonAuthPageCancel.setObjectName("pushButtonAuthPageCancel")
        self.gridLayout_4.addWidget(self.pushButtonAuthPageCancel, 0, 1, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(50, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_4.addItem(spacerItem12, 0, 0, 1, 1)
        self.stackedWidgetButtons.addWidget(self.authPageButtons)
        self.dropboxPathPageButtons = QtWidgets.QWidget()
        self.dropboxPathPageButtons.setObjectName("dropboxPathPageButtons")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.dropboxPathPageButtons)
        self.gridLayout_8.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.pushButtonDropboxPathCancel = QtWidgets.QPushButton(self.dropboxPathPageButtons)
        icon = QtGui.QIcon.fromTheme("dialog-close")
        self.pushButtonDropboxPathCancel.setIcon(icon)
        self.pushButtonDropboxPathCancel.setObjectName("pushButtonDropboxPathCancel")
        self.gridLayout_8.addWidget(self.pushButtonDropboxPathCancel, 0, 3, 1, 1)
        self.pushButtonDropboxPathUnlink = QtWidgets.QPushButton(self.dropboxPathPageButtons)
        icon = QtGui.QIcon.fromTheme("edit-clear")
        self.pushButtonDropboxPathUnlink.setIcon(icon)
        self.pushButtonDropboxPathUnlink.setObjectName("pushButtonDropboxPathUnlink")
        self.gridLayout_8.addWidget(self.pushButtonDropboxPathUnlink, 0, 1, 1, 1)
        self.pushButtonDropboxPathSelect = QtWidgets.QPushButton(self.dropboxPathPageButtons)
        icon = QtGui.QIcon.fromTheme("dialog-ok")
        self.pushButtonDropboxPathSelect.setIcon(icon)
        self.pushButtonDropboxPathSelect.setObjectName("pushButtonDropboxPathSelect")
        self.gridLayout_8.addWidget(self.pushButtonDropboxPathSelect, 0, 4, 1, 1)
        spacerItem13 = QtWidgets.QSpacerItem(50, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_8.addItem(spacerItem13, 0, 2, 1, 1)
        self.stackedWidgetButtons.addWidget(self.dropboxPathPageButtons)
        self.excludedFoldersPageButtons = QtWidgets.QWidget()
        self.excludedFoldersPageButtons.setObjectName("excludedFoldersPageButtons")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.excludedFoldersPageButtons)
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.pushButtonFolderSelectionBack = QtWidgets.QPushButton(self.excludedFoldersPageButtons)
        self.pushButtonFolderSelectionBack.setObjectName("pushButtonFolderSelectionBack")
        self.gridLayout_6.addWidget(self.pushButtonFolderSelectionBack, 0, 1, 1, 1)
        self.pushButtonFolderSelectionSelect = QtWidgets.QPushButton(self.excludedFoldersPageButtons)
        icon = QtGui.QIcon.fromTheme("dialog-ok")
        self.pushButtonFolderSelectionSelect.setIcon(icon)
        self.pushButtonFolderSelectionSelect.setObjectName("pushButtonFolderSelectionSelect")
        self.gridLayout_6.addWidget(self.pushButtonFolderSelectionSelect, 0, 2, 1, 1)
        spacerItem14 = QtWidgets.QSpacerItem(50, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_6.addItem(spacerItem14, 0, 0, 1, 1)
        self.stackedWidgetButtons.addWidget(self.excludedFoldersPageButtons)
        self.donePageButtons = QtWidgets.QWidget()
        self.donePageButtons.setObjectName("donePageButtons")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.donePageButtons)
        self.gridLayout_10.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_10.setObjectName("gridLayout_10")
        spacerItem15 = QtWidgets.QSpacerItem(50, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_10.addItem(spacerItem15, 0, 0, 1, 1)
        self.stackedWidgetButtons.addWidget(self.donePageButtons)
        self.verticalLayout.addWidget(self.stackedWidgetButtons)
        self.verticalLayout.setStretch(0, 1)

        self.retranslateUi(SetupDialog)
        self.stackedWidget.setCurrentIndex(0)
        self.stackedWidgetButtons.setCurrentIndex(0)
        self.stackedWidget.currentChanged['int'].connect(self.stackedWidgetButtons.setCurrentIndex)
        QtCore.QMetaObject.connectSlotsByName(SetupDialog)
        SetupDialog.setTabOrder(self.pushButtonLink, self.pushButtonAuthPageLink)
        SetupDialog.setTabOrder(self.pushButtonAuthPageLink, self.pushButtonAuthPageCancel)
        SetupDialog.setTabOrder(self.pushButtonAuthPageCancel, self.lineEditAuthCode)
        SetupDialog.setTabOrder(self.lineEditAuthCode, self.pushButtonDropboxPathSelect)
        SetupDialog.setTabOrder(self.pushButtonDropboxPathSelect, self.pushButtonDropboxPathCancel)
        SetupDialog.setTabOrder(self.pushButtonDropboxPathCancel, self.pushButtonDropboxPathUnlink)
        SetupDialog.setTabOrder(self.pushButtonDropboxPathUnlink, self.comboBoxDropboxPath)
        SetupDialog.setTabOrder(self.comboBoxDropboxPath, self.pushButtonFolderSelectionSelect)
        SetupDialog.setTabOrder(self.pushButtonFolderSelectionSelect, self.pushButtonFolderSelectionBack)
        SetupDialog.setTabOrder(self.pushButtonFolderSelectionBack, self.treeViewFolders)
        SetupDialog.setTabOrder(self.treeViewFolders, self.selectAllCheckBox)
        SetupDialog.setTabOrder(self.selectAllCheckBox, self.pushButtonClose)

    def retranslateUi(self, SetupDialog):
        _translate = QtCore.QCoreApplication.translate
        SetupDialog.setWindowTitle(_translate("SetupDialog", "Maestral Setup"))
        self.pushButtonLink.setText(_translate("SetupDialog", "Link Dropbox Account"))
        self.labelWelcome.setText(_translate("SetupDialog", "Welcome to Maestral, an open source Dropbox client for Linux and macOS."))
        self.labelAuthLink.setText(_translate("SetupDialog", "<html><head/><body><p align=\"center\">To link Maestral to your Dropbox account, please click <a href=\"{0}\">here</a> to retrieve an authorization token from Dropbox and enter it below.</p></body></html>"))
        self.lineEditAuthCode.setPlaceholderText(_translate("SetupDialog", "Authorization token"))
        self.labelDropboxPath.setText(_translate("SetupDialog", "<html><head/><body><p align=\"center\">Maestral has been successfully linked with your Dropbox account.</p><p align=\"center\">Please select a local folder for your Dropbox. If the folder is not empty, you will be given the option to merge its content with your remote Dropbox. Merging will not transfer or duplicate any identical files.</p><p align=\"center\">In the next step, you will be asked to choose which folders to sync.</p></body></html>"))
        self.labelFolderSelection.setText(_translate("SetupDialog", "Please select which files and folders to sync below. The inital download may take some time, depending on the size of your Dropbox."))
        self.selectAllCheckBox.setText(_translate("SetupDialog", "Select all"))
        self.labelSetupComplete.setText(_translate("SetupDialog", "<html><head/><body><p align=\"center\">You have successfully set up Maestral. </p><p align=\"center\">Please allow some time for the initial indexing and download of your Dropbox before Maestral will commence syncing.</p></body></html>"))
        self.pushButtonClose.setText(_translate("SetupDialog", "Close"))
        self.pushButtonAuthPageLink.setText(_translate("SetupDialog", "Link"))
        self.pushButtonAuthPageCancel.setText(_translate("SetupDialog", "Cancel"))
        self.pushButtonDropboxPathCancel.setText(_translate("SetupDialog", "Cancel"))
        self.pushButtonDropboxPathUnlink.setText(_translate("SetupDialog", "Unlink"))
        self.pushButtonDropboxPathSelect.setText(_translate("SetupDialog", "Select"))
        self.pushButtonFolderSelectionBack.setText(_translate("SetupDialog", "Back"))
        self.pushButtonFolderSelectionSelect.setText(_translate("SetupDialog", "Select"))
from maestral_qt.widgets import AnimatedStackedWidget, QProgressIndicator



# -*- coding: utf-8 -*-

# system imports
import os.path as osp
import time

# external packages
from PyQt6 import QtGui, QtCore, QtWidgets

# maestral modules
from maestral import __version__ as __daemon_version__
from maestral.utils.path import delete
from maestral.utils.appdirs import get_home_dir

# local imports
from . import __version__, __author__, __url__
from .selective_sync_dialog import SelectiveSyncDialog
from .bandwidth_dialog import BandwidthDialog
from .utils import (
    LINE_COLOR_DARK,
    LINE_COLOR_LIGHT,
    get_scaled_font,
    is_dark_window,
    center_window,
    icon_to_pixmap,
    get_masked_image,
    MaestralBackgroundTask,
    is_empty,
)
from .widgets import UserDialog
from .autostart import AutoStart
from .resources import (
    native_item_icon,
    APP_ICON_PATH,
    FACEHOLDER_PATH,
)
from .resources.ui_settings_window import Ui_SettingsWindow
from .resources.ui_unlink_dialog import Ui_UnlinkDialog


class UnlinkDialog(QtWidgets.QDialog, Ui_UnlinkDialog):
    # noinspection PyArgumentList
    def __init__(self, mdbx, on_unlink_complete, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.setWindowFlags(QtCore.Qt.WindowType.Sheet)
        self.setModal(True)

        self.on_unlink_complete = on_unlink_complete
        self.mdbx = mdbx

        self.buttonBox.buttons()[0].setText("Unlink")
        self.titleLabel.setFont(get_scaled_font(bold=True))
        self.infoLabel.setFont(get_scaled_font(scaling=0.9))

        icon = QtGui.QIcon(APP_ICON_PATH)
        pixmap = icon_to_pixmap(icon, self.iconLabel.width(), self.iconLabel.height())
        self.iconLabel.setPixmap(pixmap)

    def accept(self):
        self.buttonBox.setEnabled(False)
        self.progressIndicator.startAnimation()
        self.unlink_thread = MaestralBackgroundTask(
            self, self.mdbx.config_name, "unlink"
        )
        self.unlink_thread.sig_result.connect(self._on_unlink_complete)

    def _on_unlink_complete(self):
        super().accept()
        self.on_unlink_complete()


# noinspection PyArgumentList
class SettingsWindow(QtWidgets.QWidget, Ui_SettingsWindow):
    """A widget showing all of Maestral's settings."""

    _update_interval_mapping = {
        0: 60 * 60 * 24,
        1: 60 * 60 * 24 * 7,
        2: 60 * 60 * 24 * 30,
        3: 0,
    }

    def __init__(self, parent, mdbx):
        super().__init__()
        self.setupUi(self)

        self._parent = parent
        self.update_dark_mode()

        self.adjustSize()

        self.mdbx = mdbx
        self.selective_sync_dialog = SelectiveSyncDialog(self.mdbx, parent=self)
        self.bandwidth_dialog = BandwidthDialog(self.mdbx, parent=self)
        self.unlink_dialog = UnlinkDialog(self.mdbx, self.on_unlink, parent=self)
        self.autostart = AutoStart(self.mdbx.config_name)

        self.labelAccountName.setFont(get_scaled_font(1.5))
        self.labelAccountInfo.setFont(get_scaled_font(0.9))
        self.labelSpaceUsage.setFont(get_scaled_font(0.9))

        # fixes sizes of label and profile pic
        self.labelAccountName.setFixedHeight(self.labelAccountName.height())
        self._profile_pic_height = round(self.labelUserProfilePic.height() * 0.65)

        self.refresh_gui()

        # update profile pic and account info periodically
        self.update_timer = QtCore.QTimer()
        self.update_timer.timeout.connect(self.refresh_gui)

        # connect callbacks
        self.pushButtonUnlink.clicked.connect(self.unlink_dialog.exec)
        self.pushButtonExcludedFolders.clicked.connect(
            self.selective_sync_dialog.populate_folders_list
        )
        self.pushButtonExcludedFolders.clicked.connect(self.selective_sync_dialog.open)
        self.pushButtonBandwidthLimits.clicked.connect(self.bandwidth_dialog.open)
        self.checkBoxStartup.stateChanged.connect(self.on_start_on_login_clicked)
        self.checkBoxNotifications.stateChanged.connect(self.on_notifications_clicked)
        self.comboBoxUpdateInterval.currentIndexChanged.connect(
            self.on_combobox_update_interval
        )
        self.comboBoxDropboxPath.currentIndexChanged.connect(self.on_combobox_path)
        msg = (
            "Choose a new local Dropbox folder. If the new folder is not empty, you "
            "can either delete its content or merge it with your Dropbox."
        )
        self.dropbox_folder_dialog = QtWidgets.QFileDialog(self, caption=msg)
        self.dropbox_folder_dialog.setModal(True)
        self.dropbox_folder_dialog.setAcceptMode(
            QtWidgets.QFileDialog.AcceptMode.AcceptOpen
        )
        self.dropbox_folder_dialog.setFileMode(QtWidgets.QFileDialog.FileMode.Directory)
        self.dropbox_folder_dialog.setOption(QtWidgets.QFileDialog.Option.ShowDirsOnly)
        self.dropbox_folder_dialog.setLabelText(
            QtWidgets.QFileDialog.DialogLabel.Accept, "Select"
        )
        self.dropbox_folder_dialog.finished.connect(self.on_new_dbx_folder)

        center_window(self)

    def refresh_gui(self):
        # populate account info
        self.set_profile_pic_from_cache()
        self.set_account_info_from_cache()

        # populate sync section
        dbx_path = self.mdbx.dropbox_path
        folder_icon = native_item_icon(dbx_path)

        self.comboBoxDropboxPath.clear()
        self.comboBoxDropboxPath.addItem(folder_icon, self.rel_path(dbx_path))
        self.comboBoxDropboxPath.insertSeparator(1)
        self.comboBoxDropboxPath.addItem(QtGui.QIcon(), "Choose...")

        # populate app section
        self.checkBoxStartup.setChecked(self.autostart.enabled)
        self.checkBoxNotifications.setChecked(self.mdbx.notification_level <= 15)
        update_interval = self.mdbx.get_conf("app", "update_notification_interval")
        closest_key = min(
            self._update_interval_mapping,
            key=lambda x: abs(self._update_interval_mapping[x] - update_interval),
        )
        self.comboBoxUpdateInterval.setCurrentIndex(closest_key)

        # populate about section
        year = time.localtime().tm_year
        self.labelVersion.setText(
            self.labelVersion.text().format(__version__, __daemon_version__)
        )
        self.labelUrl.setText(self.labelUrl.text().format(__url__))
        self.labelCopyright.setText(self.labelCopyright.text().format(year, __author__))

    def set_profile_pic_from_cache(self):
        try:
            pixmap = get_masked_image(
                self.mdbx.account_profile_pic_path, size=self._profile_pic_height
            )
        except OSError:
            pixmap = get_masked_image(FACEHOLDER_PATH, size=self._profile_pic_height)

        self.labelUserProfilePic.setPixmap(pixmap)

    def set_account_info_from_cache(self):
        acc_display_name = self.mdbx.get_state("account", "display_name")
        acc_mail = self.mdbx.get_state("account", "email")
        acc_type = self.mdbx.get_state("account", "type")
        acc_space_usage = self.mdbx.get_state("account", "usage")
        acc_space_usage_type = self.mdbx.get_state("account", "usage_type")

        if acc_space_usage_type == "team":
            acc_space_usage += " (Team)"

        # if the display name is longer than 230 pixels, reduce font-size
        default_font = get_scaled_font(1.5)
        account_display_name_length = QtGui.QFontMetrics(
            default_font
        ).horizontalAdvance(acc_display_name)

        if account_display_name_length > 240:
            font = get_scaled_font(scaling=1.5 * 240 / account_display_name_length)
            self.labelAccountName.setFont(font)
        self.labelAccountName.setText(acc_display_name)

        if acc_type != "":
            acc_type_text = ", Dropbox {0}".format(acc_type.capitalize())
        else:
            acc_type_text = ""
        self.labelAccountInfo.setText(acc_mail + acc_type_text)
        self.labelSpaceUsage.setText(acc_space_usage)

    def on_combobox_path(self, idx):
        if idx == 2:
            initial_dir = osp.dirname(self.mdbx.dropbox_path)
            self.dropbox_folder_dialog.open()
            self.dropbox_folder_dialog.setDirectory(initial_dir)

    def on_combobox_update_interval(self, idx):
        self.mdbx.set_conf(
            "app", "update_notification_interval", self._update_interval_mapping[idx]
        )

    def on_new_dbx_folder(self, res):
        self.comboBoxDropboxPath.setCurrentIndex(0)

        if res == QtWidgets.QDialog.DialogCode.Rejected:
            return

        new_location = self.dropbox_folder_dialog.selectedFiles()[0]

        if new_location == self.mdbx.dropbox_path:
            return

        if not is_empty(new_location):
            UserDialog(
                title="Folder is not empty",
                message=(
                    f'The folder "{osp.basename(new_location)}" is not empty. '
                    "Please select an empty folder."
                ),
                parent=self,
            ).exec()

            return

        err = delete(new_location)
        if err:
            return self.on_move_completed(err)

        task = MaestralBackgroundTask(
            parent=self,
            config_name=self.mdbx.config_name,
            target="move_dropbox_directory",
            args=(new_location,),
        )

        task.sig_result.connect(self.on_move_completed)

    def on_move_completed(self, result):
        if isinstance(result, Exception):
            title = "Could not move directory"
            msg = str(result.args[0])

            msg_box = UserDialog(title, msg, parent=self)
            msg_box.open()  # no need to block with exec
            self.mdbx.start_sync()
        else:
            new_location = self.mdbx.dropbox_path
            self.comboBoxDropboxPath.setItemText(0, self.rel_path(new_location))
            self.comboBoxDropboxPath.setItemIcon(0, native_item_icon(new_location))

    def on_start_on_login_clicked(self, state):
        self.autostart.enabled = state == 2

    def on_notifications_clicked(self, state):
        self.mdbx.notification_level = 15 if state == 2 else 30

    def on_unlink(self):
        self.update_timer.stop()
        self._parent.restart()

    @staticmethod
    def rel_path(path):
        """
        Returns the path relative to the user's home directory, or the absolute
        path if not in a user directory.
        """
        home = get_home_dir()
        if osp.commonprefix([path, home]) == home:
            return osp.relpath(path, home)
        else:
            return path

    def show(self) -> None:
        self.update_timer.start(5000)
        super().show()

    def closeEvent(self, event):
        self.update_timer.stop()
        return super().closeEvent(event)

    def changeEvent(self, event):
        if event.type() == QtCore.QEvent.Type.PaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        rgb = LINE_COLOR_DARK if is_dark_window() else LINE_COLOR_LIGHT
        line_style = "color: rgb({0}, {1}, {2})".format(*rgb)

        self.line0.setStyleSheet(line_style)
        self.line1.setStyleSheet(line_style)
        self.line2.setStyleSheet(line_style)



# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/sync_issue_widget.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SyncIssueWidget(object):
    def setupUi(self, SyncIssueWidget):
        SyncIssueWidget.setObjectName("SyncIssueWidget")
        SyncIssueWidget.resize(400, 98)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SyncIssueWidget.sizePolicy().hasHeightForWidth())
        SyncIssueWidget.setSizePolicy(sizePolicy)
        SyncIssueWidget.setMinimumSize(QtCore.QSize(400, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(SyncIssueWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(SyncIssueWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setStyleSheet(".QFrame {\n"
"border: 1px solid rgb(205, 203, 205);\n"
"background-color: rgb(255, 255, 255);\n"
"border-radius: 7px;\n"
"}")
        self.frame.setFrameShape(QtWidgets.QFrame.Shape.Box)
        self.frame.setFrameShadow(QtWidgets.QFrame.Shadow.Plain)
        self.frame.setObjectName("frame")
        self.gridLayout = QtWidgets.QGridLayout(self.frame)
        self.gridLayout.setContentsMargins(7, 10, 12, 10)
        self.gridLayout.setObjectName("gridLayout")
        self.errorLabel = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.errorLabel.setFont(font)
        self.errorLabel.setWordWrap(True)
        self.errorLabel.setObjectName("errorLabel")
        self.gridLayout.addWidget(self.errorLabel, 1, 1, 1, 1)
        self.pathLabel = QElidedLabel(self.frame)
        self.pathLabel.setMaximumSize(QtCore.QSize(16777215, 20))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.pathLabel.setFont(font)
        self.pathLabel.setObjectName("pathLabel")
        self.gridLayout.addWidget(self.pathLabel, 0, 1, 1, 1)
        self.iconLabel = QtWidgets.QLabel(self.frame)
        self.iconLabel.setMinimumSize(QtCore.QSize(40, 40))
        self.iconLabel.setMaximumSize(QtCore.QSize(40, 40))
        self.iconLabel.setText("")
        self.iconLabel.setObjectName("iconLabel")
        self.gridLayout.addWidget(self.iconLabel, 0, 0, 2, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        self.actionButton = QtWidgets.QPushButton(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(30)
        sizePolicy.setVerticalStretch(30)
        sizePolicy.setHeightForWidth(self.actionButton.sizePolicy().hasHeightForWidth())
        self.actionButton.setSizePolicy(sizePolicy)
        self.actionButton.setMinimumSize(QtCore.QSize(30, 30))
        self.actionButton.setStyleSheet("QPushButton {\n"
"    border: none;\n"
"    background-color: none;\n"
"    color: rgb(68,133,243);\n"
"    font: bold\n"
"}")
        self.actionButton.setObjectName("actionButton")
        self.gridLayout.addWidget(self.actionButton, 0, 3, 2, 1)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 2, 1)
        self.verticalLayout.addWidget(self.frame)

        self.retranslateUi(SyncIssueWidget)
        QtCore.QMetaObject.connectSlotsByName(SyncIssueWidget)

    def retranslateUi(self, SyncIssueWidget):
        _translate = QtCore.QCoreApplication.translate
        SyncIssueWidget.setWindowTitle(_translate("SyncIssueWidget", "Form"))
        self.errorLabel.setText(_translate("SyncIssueWidget", "Could not upload:\n"
"Something went wrong with the job on Dropbox’s end. Please verify on the Dropbox website."))
        self.pathLabel.setText(_translate("SyncIssueWidget", "Metamorphosis.txt"))
        self.actionButton.setText(_translate("SyncIssueWidget", "•••"))
from maestral_qt.widgets import QElidedLabel



# Form implementation generated from reading ui file 'src/maestral_qt/resources/dropbox_location_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.2.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(688, 273)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setContentsMargins(-1, -1, -1, 0)
        self.gridLayout_2.setSpacing(20)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.pushButtonUnlink = QtWidgets.QPushButton(Dialog)
        self.pushButtonUnlink.setObjectName("pushButtonUnlink")
        self.gridLayout_2.addWidget(self.pushButtonUnlink, 3, 4, 1, 1)
        self.comboBoxPath = QtWidgets.QComboBox(Dialog)
        self.comboBoxPath.setObjectName("comboBoxPath")
        self.gridLayout_2.addWidget(self.comboBoxPath, 2, 1, 1, 6)
        self.pushButtonQuit = QtWidgets.QPushButton(Dialog)
        self.pushButtonQuit.setObjectName("pushButtonQuit")
        self.gridLayout_2.addWidget(self.pushButtonQuit, 3, 5, 1, 1)
        self.labelTitle = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelTitle.sizePolicy().hasHeightForWidth())
        self.labelTitle.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.labelTitle.setFont(font)
        self.labelTitle.setObjectName("labelTitle")
        self.gridLayout_2.addWidget(self.labelTitle, 0, 1, 1, 6)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 3, 0, 1, 1)
        self.pushButtonSelect = QtWidgets.QPushButton(Dialog)
        self.pushButtonSelect.setObjectName("pushButtonSelect")
        self.gridLayout_2.addWidget(self.pushButtonSelect, 3, 6, 1, 1)
        self.labelMessage = QtWidgets.QLabel(Dialog)
        self.labelMessage.setWordWrap(True)
        self.labelMessage.setObjectName("labelMessage")
        self.gridLayout_2.addWidget(self.labelMessage, 1, 1, 1, 6)
        self.labelIcon = QtWidgets.QLabel(Dialog)
        self.labelIcon.setMinimumSize(QtCore.QSize(60, 60))
        self.labelIcon.setMaximumSize(QtCore.QSize(60, 60))
        self.labelIcon.setText("")
        self.labelIcon.setObjectName("labelIcon")
        self.gridLayout_2.addWidget(self.labelIcon, 0, 0, 2, 1, QtCore.Qt.AlignmentFlag.AlignTop)
        self.spinner = QProgressIndicator(Dialog)
        self.spinner.setMinimumSize(QtCore.QSize(20, 20))
        self.spinner.setMaximumSize(QtCore.QSize(20, 20))
        self.spinner.setObjectName("spinner")
        self.gridLayout_2.addWidget(self.spinner, 3, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 3, 2, 1, 2)
        self.gridLayout.addLayout(self.gridLayout_2, 0, 1, 1, 6)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.comboBoxPath, self.pushButtonUnlink)
        Dialog.setTabOrder(self.pushButtonUnlink, self.pushButtonQuit)
        Dialog.setTabOrder(self.pushButtonQuit, self.pushButtonSelect)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        self.pushButtonUnlink.setText(_translate("Dialog", "Unlink"))
        self.pushButtonQuit.setText(_translate("Dialog", "Quit"))
        self.labelTitle.setText(_translate("Dialog", "Cannot find Dropbox folder"))
        self.pushButtonSelect.setText(_translate("Dialog", "Select"))
        self.labelMessage.setText(_translate("Dialog", "<html><head/><body><p>Your Dropbox folder has been moved or deleted from its original location. Syncing will not work until you move it back.</p><p>To move it back, click &quot;Quit&quot; below, move the Dropbox folder back to its original location, and launch Maestral again. To re-download your Dropbox, please select a new folder below.</p><p>Select &quot;Unlink&quot; to unlink your Dropbox account from Maestral.</p></body></html>"))
from maestral_qt.widgets import QProgressIndicator



# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/sync_issues_window.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SyncIssuesWindow(object):
    def setupUi(self, SyncIssuesWindow):
        SyncIssuesWindow.setObjectName("SyncIssuesWindow")
        SyncIssuesWindow.resize(425, 450)
        SyncIssuesWindow.setMinimumSize(QtCore.QSize(425, 0))
        self.horizontalLayout = QtWidgets.QHBoxLayout(SyncIssuesWindow)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.scrollArea = QtWidgets.QScrollArea(SyncIssuesWindow)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.Shape.NoFrame)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 425, 26))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setContentsMargins(0, -1, 0, -1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.horizontalLayout.addWidget(self.scrollArea)

        self.retranslateUi(SyncIssuesWindow)
        QtCore.QMetaObject.connectSlotsByName(SyncIssuesWindow)

    def retranslateUi(self, SyncIssuesWindow):
        _translate = QtCore.QCoreApplication.translate
        SyncIssuesWindow.setWindowTitle(_translate("SyncIssuesWindow", "Maestral Sync Issues"))



# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/unlink_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_UnlinkDialog(object):
    def setupUi(self, UnlinkDialog):
        UnlinkDialog.setObjectName("UnlinkDialog")
        UnlinkDialog.resize(387, 134)
        UnlinkDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(UnlinkDialog)
        self.gridLayout.setHorizontalSpacing(20)
        self.gridLayout.setObjectName("gridLayout")
        self.infoLabel = QtWidgets.QLabel(UnlinkDialog)
        self.infoLabel.setWordWrap(True)
        self.infoLabel.setObjectName("infoLabel")
        self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 2)
        self.iconLabel = QtWidgets.QLabel(UnlinkDialog)
        self.iconLabel.setMinimumSize(QtCore.QSize(60, 60))
        self.iconLabel.setMaximumSize(QtCore.QSize(60, 60))
        self.iconLabel.setText("")
        self.iconLabel.setObjectName("iconLabel")
        self.gridLayout.addWidget(self.iconLabel, 0, 0, 2, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(UnlinkDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 2, 2, 1, 1)
        self.titleLabel = QtWidgets.QLabel(UnlinkDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.titleLabel.setFont(font)
        self.titleLabel.setObjectName("titleLabel")
        self.gridLayout.addWidget(self.titleLabel, 0, 1, 1, 2)
        self.progressIndicator = QProgressIndicator(UnlinkDialog)
        self.progressIndicator.setMinimumSize(QtCore.QSize(16, 16))
        self.progressIndicator.setMaximumSize(QtCore.QSize(16, 16))
        self.progressIndicator.setObjectName("progressIndicator")
        self.gridLayout.addWidget(self.progressIndicator, 2, 1, 1, 1, QtCore.Qt.AlignmentFlag.AlignVCenter)

        self.retranslateUi(UnlinkDialog)
        self.buttonBox.accepted.connect(UnlinkDialog.accept)
        self.buttonBox.rejected.connect(UnlinkDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(UnlinkDialog)

    def retranslateUi(self, UnlinkDialog):
        _translate = QtCore.QCoreApplication.translate
        UnlinkDialog.setWindowTitle(_translate("UnlinkDialog", "Dialog"))
        self.infoLabel.setText(_translate("UnlinkDialog", "You\'ll still keep your Dropbox folder on this computer, but your files will stop syncing."))
        self.titleLabel.setText(_translate("UnlinkDialog", "Unlink your Dropbox account?"))
from maestral_qt.widgets import QProgressIndicator



# -*- coding: utf-8 -*-

# external packages
from PyQt6 import QtCore, QtWidgets, QtGui
from PyQt6.QtCore import Qt

# local imports
from .utils import get_scaled_font, icon_to_pixmap
from .utils import MaestralBackgroundTask
from .resources import APP_ICON_PATH
from .resources.ui_relink_dialog import Ui_RelinkDialog


# noinspection PyArgumentList
class RelinkDialog(QtWidgets.QDialog, Ui_RelinkDialog):
    """
    A dialog to show when Maestral's Dropbox access has expired or has been revoked.
    """

    VALID_MSG = "Verified. Restarting Maestral..."
    INVALID_MSG = "Invalid token"
    CONNECTION_ERR_MSG = "Connection failed"
    PLACEHOLDER_TEXT = "Authorization token"

    EXPIRED = 0
    REVOKED = 1

    def __init__(self, parent, reason=EXPIRED):
        super().__init__()
        self.setupUi(self)

        # noinspection PyTypeChecker
        self.setWindowFlags(
            Qt.WindowType.WindowStaysOnTopHint
            | Qt.WindowType.Sheet
            | Qt.WindowType.WindowTitleHint
            | Qt.WindowType.CustomizeWindowHint
        )

        self._parent = parent
        self.mdbx = self._parent.mdbx

        # format text labels
        if reason is self.EXPIRED:
            self.titleLabel.setText("Dropbox Access Expired")
            formatted_text = self.infoLabel.text().format(
                "has expired", self.mdbx.get_auth_url()
            )
        elif reason is self.REVOKED:
            self.titleLabel.setText("Dropbox Access Revoked")
            formatted_text = self.infoLabel.text().format(
                "has been revoked", self.mdbx.get_auth_url()
            )
        else:
            raise ValueError(
                '"reason" must be RelinkDialog.EXPIRED or ' "RelinkDialog.REVOKED."
            )
        self.infoLabel.setText(formatted_text)
        self.titleLabel.setFont(get_scaled_font(bold=True))
        self.infoLabel.setFont(get_scaled_font(scaling=0.9))

        # add app icon
        icon = QtGui.QIcon(APP_ICON_PATH)
        pixmap = icon_to_pixmap(icon, self.iconLabel.width(), self.iconLabel.height())
        self.iconLabel.setPixmap(pixmap)

        # format line edit
        self.lineEditAuthCode.setTextMargins(3, 0, 0, 0)

        # connect callbacks
        self.lineEditAuthCode.textChanged.connect(self._update_appearance)
        self.pushButtonCancel.clicked.connect(self.quit)
        self.pushButtonUnlink.clicked.connect(self.delete_creds_and_quit)
        self.pushButtonLink.clicked.connect(self.on_link_clicked)

        # other
        self.pushButtonCancel.setFocus()
        self.adjustSize()

    def quit(self):
        self.set_ui_busy()
        self._parent.quit(stop_daemon=True)

    def delete_creds_and_quit(self):
        self.set_ui_busy()
        self.mdbx.unlink()
        self._parent.quit(stop_daemon=True)

    def _update_appearance(self, text):
        placeholder_text = self.lineEditAuthCode.placeholderText()
        if text == "":
            if placeholder_text == self.PLACEHOLDER_TEXT:
                self.pushButtonLink.setEnabled(False)
                self.lineEditAuthCode.setStyleSheet("")
            elif placeholder_text == self.INVALID_MSG:
                self.pushButtonLink.setEnabled(False)
                self.lineEditAuthCode.setStyleSheet(
                    "color: rgb(205, 0, 0); font: bold;"
                )
            elif placeholder_text == self.CONNECTION_ERR_MSG:
                self.pushButtonLink.setEnabled(False)
                self.lineEditAuthCode.setStyleSheet(
                    "color: rgb(205, 0, 0); font: bold;"
                )
            elif placeholder_text == self.VALID_MSG:
                self.pushButtonLink.setEnabled(False)
                self.pushButtonUnlink.setEnabled(False)
                self.pushButtonCancel.setEnabled(False)
                self.lineEditAuthCode.setStyleSheet(
                    "color: rgb(0, 129, 0); font: bold;"
                )
        else:
            self.pushButtonLink.setEnabled(True)
            self.lineEditAuthCode.setStyleSheet("")

    def on_link_clicked(self):
        token = self.lineEditAuthCode.text()
        if token == "":
            # this should not occur because link button will be inactivate when there
            # is no text in QLineEdit
            return

        self.set_ui_busy()

        self.auth_task = MaestralBackgroundTask(
            parent=self, config_name=self.mdbx.config_name, target="link", args=(token,)
        )
        self.auth_task.sig_result.connect(self.on_link_done)

    def on_link_done(self, res):
        if res == 0:
            self.lineEditAuthCode.setPlaceholderText(self.VALID_MSG)
            QtWidgets.QApplication.processEvents()
            QtCore.QTimer.singleShot(200, self._parent.restart)
        elif res == 1:
            self.lineEditAuthCode.setPlaceholderText(self.INVALID_MSG)
            self.set_ui_idle()
        elif res == 2:
            self.lineEditAuthCode.setPlaceholderText(self.CONNECTION_ERR_MSG)
            self.set_ui_idle()

        self.lineEditAuthCode.setText("")

    def set_ui_busy(self):
        self.progressIndicator.startAnimation()
        self.lineEditAuthCode.setEnabled(False)
        self.pushButtonLink.setEnabled(False)
        self.pushButtonUnlink.setEnabled(False)
        self.pushButtonCancel.setEnabled(False)

    def set_ui_idle(self):
        self.progressIndicator.stopAnimation()
        self.lineEditAuthCode.setEnabled(True)
        self.pushButtonLink.setEnabled(True)
        self.pushButtonUnlink.setEnabled(True)
        self.pushButtonCancel.setEnabled(True)



# -*- coding: utf-8 -*-

# external packages
from PyQt6 import QtWidgets

# local imports
from .resources.ui_bandwidth_dialog import Ui_BandwidthDialog


MB_2_BYTES = 10**6


class BandwidthDialog(QtWidgets.QDialog, Ui_BandwidthDialog):
    def __init__(self, mdbx, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.setModal(True)

        self.mdbx = mdbx

        self.updateButton.clicked.connect(self.on_accepted)
        self.cancelButton.clicked.connect(self.close)

        self.radioButtonDownloadUnlimited.toggled.connect(
            self.on_limit_downloads_toggled
        )
        self.radioButtonUploadUnlimited.toggled.connect(self.on_limit_uploads_toggled)

    def update_gui(self):
        if self.mdbx.bandwidth_limit_down == 0:
            self.radioButtonDownloadUnlimited.setChecked(True)
            self.numberInputDownloadRate.setEnabled(False)
        else:
            self.radioButtonDownloadLimited.setChecked(True)
            self.numberInputDownloadRate.setValue(
                self.mdbx.bandwidth_limit_down / MB_2_BYTES
            )
            self.numberInputDownloadRate.setEnabled(True)

        if self.mdbx.bandwidth_limit_up == 0:
            self.radioButtonUploadUnlimited.setChecked(True)
            self.numberInputUploadRate.setEnabled(False)
        else:
            self.radioButtonUploadLimited.setChecked(True)
            self.numberInputUploadRate.setValue(
                self.mdbx.bandwidth_limit_up / MB_2_BYTES
            )
            self.numberInputUploadRate.setEnabled(True)

    def apply_changes(self):
        if self.radioButtonDownloadUnlimited.isChecked():
            self.mdbx.bandwidth_limit_down = 0.0
        else:
            self.mdbx.bandwidth_limit_down = (
                self.numberInputDownloadRate.value() * MB_2_BYTES
            )

        if self.radioButtonUploadUnlimited.isChecked():
            self.mdbx.bandwidth_limit_up = 0.0
        else:
            self.mdbx.bandwidth_limit_up = (
                self.numberInputUploadRate.value() * MB_2_BYTES
            )

    def on_limit_downloads_toggled(self, value: bool) -> None:
        self.numberInputDownloadRate.setEnabled(not value)

    def on_limit_uploads_toggled(self, value: bool) -> None:
        self.numberInputUploadRate.setEnabled(not value)

    def on_accepted(self, overload=None):
        self.apply_changes()
        self.accept()

    def show(self):
        self.update_gui()
        super().show()

    def open(self):
        self.update_gui()
        super().open()



# -*- coding: utf-8 -*-

# system imports
import os.path as osp
from queue import Queue

# external imports
from PyQt6 import QtGui, QtCore, QtWidgets
from PyQt6.QtCore import QModelIndex, Qt

# maestral modules
from maestral.utils.appdirs import get_home_dir
from maestral.utils.path import delete

# local imports
from .utils import MaestralBackgroundTask, icon_to_pixmap, is_empty
from .widgets import UserDialog
from .selective_sync_dialog import AsyncListFolder, FileSystemModel, DropboxPathItem
from .resources import APP_ICON_PATH, native_folder_icon
from .resources.ui_setup_dialog import Ui_SetupDialog


# noinspection PyArgumentList
class SetupDialog(QtWidgets.QDialog, Ui_SetupDialog):
    """A dialog to link and set up a new Dropbox account."""

    accepted = False

    def __init__(self, mdbx, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.mdbx = mdbx
        self.config_name = self.mdbx.config_name
        self.dbx_model = None
        self.excluded_items = []

        self.app_icon = QtGui.QIcon(APP_ICON_PATH)

        self.labelIcon_0.setPixmap(icon_to_pixmap(self.app_icon, 150))
        self.labelIcon_1.setPixmap(icon_to_pixmap(self.app_icon, 70))
        self.labelIcon_2.setPixmap(icon_to_pixmap(self.app_icon, 70))
        self.labelIcon_3.setPixmap(icon_to_pixmap(self.app_icon, 120))

        # prepare auth session
        self.auth_url = self.mdbx.get_auth_url()
        prompt = self.labelAuthLink.text().format(self.auth_url)
        self.labelAuthLink.setText(prompt)

        # set up Dropbox location combobox

        self.dropbox_location = self.mdbx.get_conf("sync", "path")

        if self.dropbox_location == "":
            folder_name = f"Dropbox ({self.config_name.capitalize()})"
            self.dropbox_location = osp.join(get_home_dir(), folder_name)

        self.comboBoxDropboxPath.addItem(native_folder_icon(), self.dropbox_location)
        self.comboBoxDropboxPath.insertSeparator(1)
        self.comboBoxDropboxPath.addItem(QtGui.QIcon(), "Choose...")
        self.comboBoxDropboxPath.currentIndexChanged.connect(self.on_combobox)

        # resize dialog buttons
        width = self.pushButtonAuthPageCancel.width() * 1.1
        width = round(width)
        for b in (
            self.pushButtonAuthPageLink,
            self.pushButtonDropboxPathUnlink,
            self.pushButtonDropboxPathSelect,
            self.pushButtonFolderSelectionBack,
            self.pushButtonFolderSelectionSelect,
            self.pushButtonAuthPageCancel,
            self.pushButtonDropboxPathCancel,
            self.pushButtonClose,
        ):
            b.setMinimumWidth(width)
            b.setMaximumWidth(width)

        self.dropbox_folder_dialog = QtWidgets.QFileDialog(self)
        self.dropbox_folder_dialog.setAcceptMode(
            QtWidgets.QFileDialog.AcceptMode.AcceptOpen
        )
        self.dropbox_folder_dialog.setFileMode(QtWidgets.QFileDialog.FileMode.Directory)
        self.dropbox_folder_dialog.setOption(
            QtWidgets.QFileDialog.Option.ShowDirsOnly, True
        )
        self.dropbox_folder_dialog.setLabelText(
            QtWidgets.QFileDialog.DialogLabel.Accept, "Select"
        )
        self.dropbox_folder_dialog.setDirectory(get_home_dir())
        self.dropbox_folder_dialog.fileSelected.connect(self.on_new_dbx_folder)
        self.dropbox_folder_dialog.rejected.connect(
            lambda: self.comboBoxDropboxPath.setCurrentIndex(0)
        )

        # connect buttons to callbacks
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.pushButtonLink.clicked.connect(self.on_link_clicked)
        self.pushButtonAuthPageCancel.clicked.connect(self.on_reject_requested)
        self.pushButtonAuthPageLink.clicked.connect(self.on_auth_clicked)
        self.pushButtonDropboxPathCancel.clicked.connect(self.on_reject_requested)
        self.pushButtonDropboxPathSelect.clicked.connect(
            self.on_dropbox_location_selected
        )
        self.pushButtonDropboxPathUnlink.clicked.connect(self.unlink_and_go_to_start)
        self.pushButtonFolderSelectionBack.clicked.connect(
            self.stackedWidget.slideInPrev
        )
        self.pushButtonFolderSelectionSelect.clicked.connect(self.on_folders_selected)
        self.pushButtonClose.clicked.connect(self.on_accept_requested)
        self.selectAllCheckBox.clicked.connect(self.on_select_all_clicked)

        if not self.mdbx.pending_link:
            self.stackedWidget.setCurrentIndex(2)

    # =============================================================================
    # Main callbacks
    # =============================================================================

    def closeEvent(self, event):
        if self.stackedWidget.currentIndex == 4:
            self.on_accept_requested()
        else:
            self.on_reject_requested()

    def on_accept_requested(self):
        del self.mdbx

        self.accepted = True
        self.accept()

    def on_reject_requested(self):
        self.accepted = False
        self.reject()

    def unlink_and_go_to_start(self):
        self.mdbx.unlink()
        self.stackedWidget.slideInIdx(0)

    def on_link_clicked(self):
        self.stackedWidget.fadeInIdx(1)
        self.pushButtonAuthPageLink.setFocus()

    def on_auth_clicked(self):
        if self.lineEditAuthCode.text() == "":
            msg = "Please enter an authentication token."
            msg_box = UserDialog("Authentication failed.", msg, parent=self)
            msg_box.open()
        else:
            self.progressIndicator.startAnimation()
            self.pushButtonAuthPageLink.setEnabled(False)
            self.lineEditAuthCode.setEnabled(False)

            self.link_async()

    def link_async(self):
        token = self.lineEditAuthCode.text()

        self.auth_task = MaestralBackgroundTask(
            parent=self, config_name=self.mdbx.config_name, target="link", args=(token,)
        )
        self.auth_task.sig_result.connect(self.on_link_done)

    def on_link_done(self, res):
        if res == 0:
            # switch to next page
            self.stackedWidget.slideInIdx(2)
            self.pushButtonDropboxPathSelect.setFocus()
            self.lineEditAuthCode.clear()  # clear since we might come back on unlink

        elif res == 1:
            msg = "Please make sure that you entered the correct authentication token."
            msg_box = UserDialog("Authentication failed.", msg, parent=self)
            msg_box.open()
        elif res == 2:
            msg = (
                "Please make sure that you are connected to the internet and try again."
            )
            msg_box = UserDialog("Connection failed.", msg, parent=self)
            msg_box.open()

        self.progressIndicator.stopAnimation()
        self.pushButtonAuthPageLink.setEnabled(True)
        self.lineEditAuthCode.setEnabled(True)

    def on_dropbox_location_selected(self):
        # start with clean sync state
        self.mdbx.reset_sync_state()

        # apply dropbox path
        try:
            if osp.exists(self.dropbox_location):
                if is_empty(self.dropbox_location):
                    delete(self.dropbox_location, raise_error=True)
                else:
                    msg_box = UserDialog(
                        title="Folder is not empty",
                        message=(
                            f'The folder "{osp.basename(self.dropbox_location)}" is '
                            "not empty. Would you like to merge its content with your "
                            "Dropbox?"
                        ),
                        button_names=("Cancel", "Merge"),
                        parent=self,
                    )
                    res = msg_box.exec()

                    if res == UserDialog.DialogCode.Accepted:
                        return
                    elif res == UserDialog.DialogCode.Rejected:
                        pass

            self.mdbx.create_dropbox_directory(self.dropbox_location)
        except OSError:
            msg_box = UserDialog(
                title="Could not set directory",
                message=(
                    "Please check if you have permissions to write to the "
                    "selected location."
                ),
                parent=self,
            )
            msg_box.exec()
            return

        # switch to next page
        self.mdbx.set_conf("sync", "excluded_items", [])
        self.stackedWidget.slideInIdx(3)
        self.treeViewFolders.setFocus()

        # populate folder list
        if not self.excluded_items:  # don't repopulate
            self.populate_folders_list()

    def on_folders_selected(self):
        self.mdbx.excluded_items = self.get_excluded_items()

        # if any excluded items are currently on the drive, delete them
        for item in self.excluded_items:
            local_item = self.mdbx.to_local_path(item)
            delete(local_item)

        # switch to next page
        self.stackedWidget.slideInIdx(4)

    # =============================================================================
    # Helper functions
    # =============================================================================

    def on_combobox(self, idx):
        if idx == 2:
            self.dropbox_folder_dialog.open()

    def on_new_dbx_folder(self, new_location):
        self.comboBoxDropboxPath.setCurrentIndex(0)
        if not new_location == "":
            self.comboBoxDropboxPath.setItemText(0, new_location)

        self.dropbox_location = new_location

    def populate_folders_list(self):
        self.pushButtonFolderSelectionSelect.setEnabled(False)

        self.async_loader = AsyncListFolder(self.mdbx.config_name, self)
        self.dbx_root = DropboxPathItem(
            self.async_loader, set(self.mdbx.excluded_items)
        )
        self.dbx_model = FileSystemModel(self.dbx_root)
        self.dbx_model.dataChanged.connect(self.update_select_all_checkbox)
        self.treeViewFolders.setModel(self.dbx_model)

        self.dbx_model.loading_done.connect(
            lambda: self.pushButtonFolderSelectionSelect.setEnabled(True)
        )
        self.dbx_model.loading_failed.connect(
            lambda: self.pushButtonFolderSelectionSelect.setEnabled(False)
        )

        self.dbx_model.loading_done.connect(
            lambda: self.selectAllCheckBox.setEnabled(True)
        )
        self.dbx_model.loading_failed.connect(
            lambda: self.selectAllCheckBox.setEnabled(False)
        )

        self.dbx_model.loading_done.connect(
            lambda: self.treeViewFolders.resizeColumnToContents(0)
        )

    def update_select_all_checkbox(self):
        check_states = []
        for irow in range(self.dbx_model._root_item.child_count_loaded()):
            index = self.dbx_model.index(irow, 1, QModelIndex())
            check_states.append(
                self.dbx_model.data(index, Qt.ItemDataRole.CheckStateRole)
            )
        if all(cs == 2 for cs in check_states):
            self.selectAllCheckBox.setChecked(True)
        else:
            self.selectAllCheckBox.setChecked(False)

    def on_select_all_clicked(self, checked):
        checked_state = 2 if checked else 0
        for irow in range(self.dbx_model._root_item.child_count_loaded()):
            index = self.dbx_model.index(irow, 1, QModelIndex())
            self.dbx_model.setCheckState(index, checked_state)

    def get_excluded_items(self):
        # We start with an empty excluded list since this is the initial setup.
        # We add unchecked items to the excluded list.
        excluded_items = []

        queue = Queue()
        queue.put(self.dbx_model._root_item)

        while not queue.empty():
            node = queue.get()

            if node.checkState == 0:
                excluded_items.append(node._path_lower)

            for child in node._children:
                if isinstance(child, DropboxPathItem):
                    queue.put(child)

        return excluded_items

    def changeEvent(self, event):
        if event.type() == QtCore.QEvent.Type.PaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        if self.dbx_model:
            self.dbx_model.reloadData(
                [Qt.ItemDataRole.DecorationRole]
            )  # reload folder icons

    # static method to create the dialog and return Maestral instance on success
    @staticmethod
    def configureMaestral(mdbx, parent=None):
        fsd = SetupDialog(mdbx, parent)
        fsd.show()
        fsd.exec()

        return fsd.accepted



# -*- coding: utf-8 -*-

# external packages
import markdown2
from PyQt6 import QtWidgets, QtGui, QtCore
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QPixmap, QPainter

# local imports
from . import __url__
from .resources import APP_ICON_PATH
from .utils import (
    get_scaled_font,
    icon_to_pixmap,
    center_window,
    is_dark_window,
    IS_MACOS,
)


_USER_DIALOG_ICON_SIZE = 60


# ======================================================================================
# Dialogs
# ======================================================================================


# noinspection PyArgumentList, PyTypeChecker, PyCallByClass
class BackgroundTaskProgressDialog(QtWidgets.QDialog):
    """A progress dialog to show during long-running background tasks."""

    def __init__(self, title, message="", cancel=True, parent=None, width=450):
        super().__init__(parent=parent)
        self.setModal(True)
        self.setWindowModality(Qt.WindowModality.WindowModal)
        self.setWindowFlags(
            Qt.WindowType.WindowStaysOnTopHint
            | Qt.WindowType.Sheet
            | Qt.WindowType.WindowTitleHint
            | Qt.WindowType.CustomizeWindowHint
        )
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowTitle("")
        self.setFixedWidth(width)

        self.gridLayout = QtWidgets.QGridLayout()
        self.setLayout(self.gridLayout)

        self.iconLabel = QtWidgets.QLabel(self)
        self.titleLabel = QtWidgets.QLabel(self)
        self.infoLabel = QtWidgets.QLabel(self)
        self.progressBar = QtWidgets.QProgressBar()
        self.buttonBox = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.StandardButton.Cancel
        )

        self.iconLabel.setMinimumSize(_USER_DIALOG_ICON_SIZE, _USER_DIALOG_ICON_SIZE)
        self.iconLabel.setMaximumSize(_USER_DIALOG_ICON_SIZE, _USER_DIALOG_ICON_SIZE)
        self.iconLabel.setAlignment(Qt.AlignmentFlag.AlignTop)
        self.titleLabel.setFont(get_scaled_font(bold=True))
        self.infoLabel.setFont(get_scaled_font(scaling=0.9))
        self.infoLabel.setFixedWidth(width - 150)
        self.infoLabel.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.MinimumExpanding,
            QtWidgets.QSizePolicy.Policy.MinimumExpanding,
        )
        self.infoLabel.setWordWrap(True)
        self.infoLabel.setOpenExternalLinks(True)

        icon = QtGui.QIcon(APP_ICON_PATH)
        self.iconLabel.setPixmap(icon_to_pixmap(icon, _USER_DIALOG_ICON_SIZE))
        self.titleLabel.setText(title)
        self.infoLabel.setText(message)

        self.buttonBox.rejected.connect(self.reject)
        self.progressBar.setMinimum(0)
        self.progressBar.setMaximum(0)

        self.gridLayout.addWidget(self.iconLabel, 0, 0, 3, 1)
        self.gridLayout.addWidget(self.titleLabel, 0, 1, 1, 1)

        if message:
            self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 1)
            self.gridLayout.addWidget(self.progressBar, 2, 1, 1, 1)
        else:
            self.gridLayout.addWidget(self.progressBar, 1, 1, 1, 1)

        if message and cancel:
            self.gridLayout.addWidget(self.buttonBox, 3, 1, -1, -1)
        elif cancel:
            self.gridLayout.addWidget(self.buttonBox, 2, 1, -1, -1)

        self.adjustSize()
        center_window(self)


# noinspection PyArgumentList,PyTypeChecker
class UserDialog(QtWidgets.QDialog):
    """
    A template user dialog for Maestral. Shows a traceback if given in constructor.
    """

    MINIMUM_BUTTON_SIZE = 85

    Accepted2 = 2

    def __init__(
        self,
        title,
        message,
        details=None,
        checkbox=None,
        parent=None,
        button_names=("Ok",),
    ):
        """
        A user dialog for Maestral.

        :param str title: Title of dialog.
        :param str message: Message.
        :param str details: Optional details to show in a text view.
        :param str checkbox: Optional checkbox to show above dialog buttons.
        :param QtWidget parent: Parent.
        """
        super().__init__(parent=parent)
        self.setModal(True)
        self.setWindowModality(Qt.WindowModality.WindowModal)
        self.setWindowFlags(
            Qt.WindowType.WindowStaysOnTopHint
            | Qt.WindowType.Sheet
            | Qt.WindowType.WindowTitleHint
            | Qt.WindowType.CustomizeWindowHint
        )
        self.setWindowTitle("")

        self.gridLayout = QtWidgets.QGridLayout()
        self.setLayout(self.gridLayout)

        self.iconLabel = QtWidgets.QLabel(self)
        self.titleLabel = QtWidgets.QLabel(self)
        self.infoLabel = QtWidgets.QLabel(self)

        self.iconLabel.setMinimumSize(_USER_DIALOG_ICON_SIZE, _USER_DIALOG_ICON_SIZE)
        self.iconLabel.setMaximumSize(_USER_DIALOG_ICON_SIZE, _USER_DIALOG_ICON_SIZE)
        self.gridLayout.setHorizontalSpacing(self.gridLayout.horizontalSpacing() * 2)
        self.gridLayout.setVerticalSpacing(self.gridLayout.verticalSpacing() * 2)
        self.titleLabel.setFont(get_scaled_font(bold=True))
        self.infoLabel.setFont(get_scaled_font(scaling=0.9))
        self.infoLabel.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.MinimumExpanding,
            QtWidgets.QSizePolicy.Policy.MinimumExpanding,
        )
        self.infoLabel.setWordWrap(True)
        self.infoLabel.setOpenExternalLinks(True)

        icon = QtGui.QIcon(APP_ICON_PATH)
        self.iconLabel.setPixmap(icon_to_pixmap(icon, _USER_DIALOG_ICON_SIZE))
        self.titleLabel.setText(title)
        self.infoLabel.setText(message)

        if details:
            self.details = QtWidgets.QTextBrowser(self)
            self.details.setText(details)
            self.details.setOpenExternalLinks(True)

        if checkbox:
            self.checkbox = QtWidgets.QCheckBox(checkbox)

        self.buttonBox = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.StandardButton.Ok
        )
        self.buttonBox.accepted.connect(self.accept)

        self.gridLayout.addWidget(self.iconLabel, 0, 0, 2, 1, Qt.AlignmentFlag.AlignTop)
        self.gridLayout.addWidget(self.titleLabel, 0, 1, 1, 1)
        self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 1)
        if details:
            self.gridLayout.addWidget(self.details, 2, 1, 1, 1)
            self.gridLayout.setRowStretch(2, 1)
        if checkbox:
            self.gridLayout.addWidget(self.checkbox, 3, 1, 1, 1)
        self.gridLayout.addWidget(self.buttonBox, 4, 1, -1, -1)

        if len(button_names) < 1:
            ValueError("Dialog must have at least one button")

        self.setAcceptButtonName(button_names[0])

        if len(button_names) > 1:
            self.addCancelButton(name=button_names[1])

        if len(button_names) > 2:
            self.addSecondAcceptButton(name=button_names[2])

        if len(button_names) > 3:
            ValueError("Dialog cannot have more than three buttons")

        self.setWidth(700 if details else 450)
        self.adjustSize()
        center_window(self)

        for button in self.buttonBox.buttons():
            if button.sizeHint().width() < self.MINIMUM_BUTTON_SIZE:
                button.setFixedWidth(self.MINIMUM_BUTTON_SIZE)

    def setWidth(self, width):
        self.setFixedWidth(width)
        self.infoLabel.setFixedWidth(width - 150)

    def setAcceptButtonName(self, name):
        self.buttonBox.buttons()[0].setText(name)

    def setAcceptButtonIcon(self, icon):
        if isinstance(icon, QtGui.QIcon):
            self.buttonBox.buttons()[0].setIcon(icon)
        elif isinstance(icon, str):
            self.buttonBox.buttons()[0].setIcon(QtGui.QIcon.fromTheme(icon))

    def addCancelButton(self, name="Cancel", icon=None):
        self._cancelButton = self.buttonBox.addButton(
            QtWidgets.QDialogButtonBox.StandardButton.Cancel
        )
        self._cancelButton.setText(name)
        if isinstance(icon, QtGui.QIcon):
            self._cancelButton.setIcon(icon)
        elif isinstance(icon, str):
            self._cancelButton.setIcon(QtGui.QIcon.fromTheme(icon))
        self._cancelButton.clicked.connect(self.close)

    def setCancelButtonName(self, name):
        self._cancelButton.setText(name)

    def addSecondAcceptButton(self, name, icon="dialog-ok"):
        self._acceptButton2 = self.buttonBox.addButton(
            QtWidgets.QDialogButtonBox.StandardButton.Ignore
        )
        self._acceptButton2.setText(name)
        if isinstance(icon, QtGui.QIcon):
            self._acceptButton2.setIcon(icon)
        elif isinstance(icon, str):
            self._acceptButton2.setIcon(QtGui.QIcon.fromTheme(icon))
        self._acceptButton2.clicked.connect(lambda: self.setResult(self.Accepted2))
        self._acceptButton2.clicked.connect(self.close)

    def setSecondAcceptButtonName(self, name):
        self._acceptButton2.setText(name)


# dialog launch helpers


def show_dialog(title, message, details=None, level="info"):
    UserDialog(title, message, details).exec()


def show_stacktrace_dialog(traceback):
    title = "An unexpected error occurred"
    message = (
        "You can report this issue together with the traceback below on GitHub. "
        "Please restart Maestral to continue syncing."
    )
    error_dialog = UserDialog(
        title,
        message,
        details=traceback,
        button_names=("Close",),
    )
    error_dialog.exec()


def show_update_dialog(latest_release, release_notes_md):
    url = f"{__url__}/download"
    message = (
        "Maestral v{0} is available. Please use your package manager to "
        'update Maestral or go to the <a href="{1}">releases</span></a> '
        "page to download the new version. "
        '<div style="height:5px;font-size:5px;">&nbsp;<br></div>'
        "<b>Release notes:</b>"
    ).format(latest_release, url)
    release_notes_html = markdown2.markdown(release_notes_md)
    list_style = (
        '<ul style="margin-top: 0px; margin-bottom: 0px; margin-left: -20px; '
        'margin-right: 0px; -qt-list-indent: 1;">'
    )
    styled_release_notes = release_notes_html.replace("<ul>", list_style)
    update_dialog = UserDialog("Update available", message, styled_release_notes)
    update_dialog.exec()


# ======================================================================================
# Animation widgets
# ======================================================================================


# noinspection PyArgumentList
class FaderWidget(QtWidgets.QWidget):
    pixmap_opacity = 1.0

    def __init__(self, old_widget, new_widget, duration=300):
        super().__init__(new_widget)

        pr = QtWidgets.QApplication.instance().devicePixelRatio()
        self.old_pixmap = QPixmap(new_widget.size() * pr)
        self.old_pixmap.setDevicePixelRatio(pr)
        old_widget.render(self.old_pixmap)

        self.timeline = QtCore.QTimeLine()
        self.timeline.valueChanged.connect(self.animate)
        self.timeline.finished.connect(self.close)
        self.timeline.setDuration(duration)
        self.timeline.start()

        self.resize(new_widget.size())
        self.show()

    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self)
        painter.setOpacity(self.pixmap_opacity)
        painter.drawPixmap(0, 0, self.old_pixmap)
        painter.end()

    def animate(self, value):
        self.pixmap_opacity = 1.0 - value
        self.repaint()


# noinspection PyArgumentList
class AnimatedStackedWidget(QtWidgets.QStackedWidget):
    """
    A subclass of ``QStackedWidget`` with sliding or fading animations between stacks.
    """

    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.m_direction = Qt.Orientation.Horizontal
        self.m_speed = 300
        self.m_animationtype = QtCore.QEasingCurve.Type.OutCubic
        self.m_now = 0
        self.m_next = 0
        self.m_wrap = False
        self.m_pnow = QtCore.QPoint(0, 0)
        self.m_active = False

    def setDirection(self, direction):
        self.m_direction = direction

    def setSpeed(self, speed):
        self.m_speed = speed

    def setAnimation(self, animationtype):
        self.m_animationtype = animationtype

    def setWrap(self, wrap):
        self.m_wrap = wrap

    def slideInPrev(self):
        now = self.currentIndex()
        if self.m_wrap or now > 0:
            self.slideInIdx(now - 1)

    def slideInNext(self):
        now = self.currentIndex()
        if self.m_wrap or now < (self.count() - 1):
            self.slideInIdx(now + 1)

    def slideInIdx(self, idx):
        if idx > (self.count() - 1):
            idx %= self.count()
        elif idx < 0:
            idx = (idx + self.count()) % self.count()
        self.slideInWgt(self.widget(idx))

    def slideInWgt(self, newwidget):
        if self.m_active:
            return

        self.m_active = True

        _now = self.currentIndex()
        _next = self.indexOf(newwidget)

        if _now == _next:
            self.m_active = False
            return

        offsetx, offsety = self.frameRect().width(), self.frameRect().height()
        self.widget(_next).setGeometry(self.frameRect())

        if not self.m_direction == Qt.Orientation.Horizontal:
            if _now < _next:
                offsetx, offsety = 0, -offsety
            else:
                offsetx = 0
        else:
            if _now < _next:
                offsetx, offsety = -offsetx, 0
            else:
                offsety = 0

        pnext = self.widget(_next).pos()
        pnow = self.widget(_now).pos()
        self.m_pnow = pnow

        offset = QtCore.QPoint(offsetx, offsety)
        self.widget(_next).move(pnext - offset)
        self.widget(_next).show()
        self.widget(_next).raise_()

        anim_group = QtCore.QParallelAnimationGroup(
            self, finished=self.animationDoneSlot
        )

        for index, start, end in zip(
            (_now, _next), (pnow, pnext - offset), (pnow + offset, pnext)
        ):
            animation = QtCore.QPropertyAnimation(
                self.widget(index),
                b"pos",
                duration=self.m_speed,
                easingCurve=self.m_animationtype,
                startValue=start,
                endValue=end,
            )
            anim_group.addAnimation(animation)

        self.m_next = _next
        self.m_now = _now
        self.m_active = True
        anim_group.start(QtCore.QAbstractAnimation.DeletionPolicy.DeleteWhenStopped)

    def animationDoneSlot(self):
        self.setCurrentIndex(self.m_next)
        self.widget(self.m_now).hide()
        self.widget(self.m_now).move(self.m_pnow)
        self.m_active = False

    def fadeInIdx(self, index):
        self.fader_widget = FaderWidget(
            self.currentWidget(), self.widget(index), self.m_speed
        )
        self.setCurrentIndex(index)


# ======================================================================================
# Misc
# ======================================================================================


# noinspection PyArgumentList
class QProgressIndicator(QtWidgets.QWidget):
    """
    A macOS style spinning progress indicator. ``QProgressIndicator`` automatically
    detects and adjusts to 'dark mode' appearances.
    """

    m_angle = None
    m_timerId = None
    m_delay = None
    m_displayedWhenStopped = None
    m_color = None
    m_light_color = QtGui.QColor(170, 170, 170)
    m_dark_color = QtGui.QColor(40, 40, 40)

    def __init__(self, parent=None):
        # Call parent class constructor first
        super().__init__(parent)

        # Initialize instance variables
        self.m_angle = 0
        self.m_timerId = -1
        self.m_delay = 5 / 60 * 1000
        self.m_displayedWhenStopped = False
        self.m_color = self.m_dark_color

        self.update_dark_mode()

        # Set size and focus policy
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed
        )
        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)

    def animationDelay(self):
        return self.m_delay

    def isAnimated(self):
        return self.m_timerId != -1

    def isDisplayedWhenStopped(self):
        return self.m_displayedWhenStopped

    def getColor(self):
        return self.m_color

    def sizeHint(self):
        return QtCore.QSize(20, 20)

    def startAnimation(self):
        self.m_angle = 0

        if self.m_timerId == -1:
            self.m_timerId = self.startTimer(int(self.m_delay))

    def stopAnimation(self):
        if self.m_timerId != -1:
            self.killTimer(self.m_timerId)

        self.m_timerId = -1
        self.update()

    def setAnimationDelay(self, delay):
        if self.m_timerId != -1:
            self.killTimer(self.m_timerId)

        self.m_delay = delay

        if self.m_timerId != -1:
            self.m_timerId = self.startTimer(self.m_delay)

    def setDisplayedWhenStopped(self, state):
        self.m_displayedWhenStopped = state
        self.update()

    def setColor(self, color):
        self.m_color = color
        self.update()

    def timerEvent(self, event):
        self.m_angle = (self.m_angle + 30) % 360
        self.update()

    def paintEvent(self, event):
        if (not self.m_displayedWhenStopped) and (not self.isAnimated()):
            return

        width = min(self.width(), self.height())

        painter = QPainter(self)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        outerRadius = (width - 1) * 0.5
        innerRadius = (width - 1) * 0.5 * 0.4375

        capsuleHeight = outerRadius - innerRadius
        capsuleWidth = width * 3 / 32
        capsuleRadius = capsuleWidth / 2

        for i in range(0, 12):
            color = QtGui.QColor(self.m_color)

            if self.isAnimated():
                color.setAlphaF(1.0 - (i / 12.0))
            else:
                color.setAlphaF(0.2)

            painter.setPen(Qt.PenStyle.NoPen)
            painter.setBrush(color)
            painter.save()
            painter.translate(self.rect().center())
            painter.rotate(self.m_angle - (i * 30.0))

            width = -1 * capsuleWidth / 2
            height = -1 * (innerRadius + capsuleHeight)

            painter.drawRoundedRect(
                round(width),
                round(height),
                round(capsuleWidth),
                round(capsuleHeight),
                capsuleRadius,
                capsuleRadius,
            )
            painter.restore()

    def changeEvent(self, event):
        if event.type() == QtCore.QEvent.Type.PaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        if is_dark_window():
            self.setColor(self.m_light_color)
        else:
            self.setColor(self.m_dark_color)


class CustomCombobox(QtWidgets.QComboBox):
    def paintEvent(self, e):
        painter = QtWidgets.QStylePainter(self)
        painter.setPen(self.palette().color(QtGui.QPalette.ColorRole.Text))

        opt = QtWidgets.QStyleOptionComboBox()
        self.initStyleOption(opt)

        if IS_MACOS:
            # see QTBUG-78727 and QTBUG-78727
            opt.rect.adjust(-2, 0, 2, 0)

        painter.drawComplexControl(QtWidgets.QStyle.ComplexControl.CC_ComboBox, opt)
        painter.drawControl(QtWidgets.QStyle.ControlElement.CE_ComboBoxLabel, opt)


class QElidedLabel(QtWidgets.QLabel):

    """A QLabel with elided text

    Eliding is loosely based on
    http://gedgedev.blogspot.ch/2010/12/elided-labels-in-qt.html

    """

    def __init__(self, parent=None, elidemode=Qt.TextElideMode.ElideRight):
        super().__init__(parent)
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self._elidemode = elidemode
        self._elided_text = ""

    def _update_elided_text(self, width):
        """Update the elided text when necessary.

        Args:
            width: The maximal width the text should take.
        """
        if self.text():
            self._elided_text = self.fontMetrics().elidedText(
                self.text(), self._elidemode, width, Qt.TextFlag.TextShowMnemonic
            )
        else:
            self._elided_text = ""

    def elideMode(self):
        """Returns the current elide mode."""
        return self._elidemode

    def setElideMode(self, elidemode):
        """Sets the elide mode and updates the appearance."""
        self._elidemode = elidemode
        self._update_elided_text(self.geometry().width())

    def setText(self, txt):
        """Extend QLabel::setText to update the elided text afterwards.

        Args:
            txt: The text to set (string).
        """
        super().setText(txt)
        if self._elidemode != Qt.TextElideMode.ElideNone:
            self._update_elided_text(self.geometry().width())

    def resizeEvent(self, e):
        """Extend QLabel::resizeEvent to update the elided text afterwards."""
        super().resizeEvent(e)
        size = e.size()
        self._update_elided_text(size.width())

    def paintEvent(self, e):
        """Override QLabel::paintEvent to draw elided text."""
        if self._elidemode == Qt.TextElideMode.ElideNone:
            super().paintEvent(e)
        else:
            e.accept()
            painter = QPainter(self)
            geom = self.geometry()
            painter.drawText(
                0,
                0,
                geom.width(),
                geom.height(),
                int(self.alignment()),
                self._elided_text,
            )



# -*- coding: utf-8 -*-

# system imports
import os
import threading
from queue import Queue

# external packages
from PyQt6 import QtCore, QtWidgets, QtGui
from PyQt6.QtCore import QAbstractItemModel, QModelIndex, Qt
from PyQt6.QtCore import pyqtSignal as Signal

# maestral modules
from maestral.daemon import MaestralProxy
from maestral.exceptions import NotAFolderError, NotFoundError, BusyError
from maestral.utils.path import is_child, is_equal_or_child
from maestral.core import FolderMetadata

# local imports
from .utils import BackgroundTask
from .widgets import UserDialog
from .resources import native_folder_icon, native_file_icon
from .resources.ui_selective_sync_dialog import Ui_SelectiveSyncDialog


# noinspection PyTypeChecker
class FileSystemModel(QAbstractItemModel):
    """A QAbstractItemModel which loads items and their children on-demand and
    asynchronously. It is useful for displaying a hierarchy from a source which is
    slow to load (remote server, slow file system, etc)."""

    loading_failed = Signal()
    loading_done = Signal()

    def __init__(self, root, parent=None, checkbox_column=1):
        super().__init__(parent=parent)
        self._root_item = root
        self._root_item.loading_done.connect(self.reloadData)
        self._root_item.loading_failed.connect(self.on_loading_failed)
        self._header = self._root_item.header()
        self._flags = Qt.ItemFlag.ItemIsUserCheckable
        self.checkbox_column = checkbox_column

    def on_loading_failed(self):
        self.display_message(
            "Could not connect to Dropbox. Please check your internet connection."
        )

    def display_message(self, message):
        self._root_item._children = [MessageTreeItem(self._root_item, message=message)]

        self.loading_failed.emit()
        self.modelReset.emit()

    def reloadData(self, roles=None):
        if not roles:
            roles = [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.CheckStateRole]

        self.dataChanged.emit(QModelIndex(), QModelIndex(), roles)
        self.layoutChanged.emit()
        self.loading_done.emit()

    def flags(self, index):
        flags = super().flags(index) | self._flags
        return flags

    def columnCount(self, parent=None):
        if parent and parent.isValid():
            return parent.internalPointer().column_count()
        else:
            return len(self._header)

    def rowCount(self, parent=QModelIndex()):
        if parent.column() > 0:
            return 0
        if not parent.isValid():
            parent_item = self._root_item
        else:
            parent_item = parent.internalPointer()
        return parent_item.child_count()

    def hasChildren(self, parent=None):
        if parent and parent.isValid():
            return parent.internalPointer().can_have_children
        else:
            return True

    def checkState(self, index):
        if not index.isValid():
            return None
        else:
            return index.internalPointer().checkState

    def setCheckState(self, index, value):
        if index.isValid():
            item = index.internalPointer()
            item.checkState = value
            self.dataChanged.emit(index, index)
            self.layoutChanged.emit()
            return True
        return False

    def setData(self, index, value, role):
        if (
            role == Qt.ItemDataRole.CheckStateRole
            and index.column() == self.checkbox_column
        ):
            self.setCheckState(index, value)
            return True

        return super().setData(index, value, role)

    def data(self, index, role):
        if not index.isValid():
            return None
        item = index.internalPointer()
        column = index.column()
        if role == Qt.ItemDataRole.DisplayRole:
            return item.data(column)
        if role == Qt.ItemDataRole.CheckStateRole and column == self.checkbox_column:
            return item.checkState
        if role == Qt.ItemDataRole.DecorationRole and column == 0:
            return item.icon
        return None

    def headerData(self, column, orientation, role):
        if (
            orientation == Qt.Orientation.Horizontal
            and role == Qt.ItemDataRole.DisplayRole
        ):
            try:
                return self._header[column]
            except IndexError:
                pass
        return None

    def index(self, row, column, parent):
        if not self.hasIndex(row, column, parent):
            return QModelIndex()
        if not parent.isValid():
            parent_item = self._root_item
        else:
            parent_item = parent.internalPointer()
        child_item = parent_item.child_at(row)
        if child_item:
            return self.createIndex(row, column, child_item)
        else:
            return QModelIndex()

    def parent(self, index):
        if not index.isValid():
            return QModelIndex()
        child_item = index.internalPointer()
        if not child_item:
            return QModelIndex()
        parent_item = child_item.parent_()
        if parent_item == self._root_item:
            return QModelIndex()
        return self.createIndex(parent_item.row(), 0, parent_item)

    def sort(self, column, order):
        self.layoutAboutToBeChanged.emit()
        self._root_item.sort(column, order)
        self.dataChanged.emit(QModelIndex(), QModelIndex())
        self.layoutChanged.emit()


class AbstractTreeItem(QtCore.QObject):
    """
    An abstract item for `TreeModel`. To be subclassed depending on the application.
    """

    loading_done = Signal()
    loading_failed = Signal()

    def __init__(self, parent=None):
        QtCore.QObject.__init__(self, parent=parent)
        self._children = []
        self._parent = parent
        self._children_update_started = False
        self._children_update_completed = False
        self._checkStateChanged = False

        if self._parent:
            self.loading_done.connect(self._parent.loading_done)
            self.loading_failed.connect(self._parent.loading_failed)

        self.icon = QtGui.QIcon()
        self._checkState = 0
        self.can_have_children = True

    @property
    def checkState(self):
        return self._checkState

    @checkState.setter
    def checkState(self, state):
        self._checkStateChanged = True
        self._checkState = state

        self._checkStatePropagateToChildren(state)
        self._checkStatePropagateToParent(state)

    def _checkStatePropagateToChildren(self, state):
        pass

    def _checkStatePropagateToParent(self, state):
        pass

    def header(self):
        # subclass this
        raise NotImplementedError(self.header)

    def column_count(self):
        return len(self.header())

    def parent_(self):
        return self._parent

    def _async_loading_done(self, result):
        raise NotImplementedError(self._async_loading_done)

    def _create_children_async(self):
        raise NotImplementedError(self._create_children_async)

    def row(self):
        if self._parent:
            return self._parent._children.index(self)
        return 0

    def children_(self):
        if not self._children_update_started:
            self._create_children_async()
            self._children_update_started = True
        return self._children

    def child_at(self, row):
        return self.children_()[row]

    def data(self, column):
        # subclass this
        raise NotImplementedError(self.data)

    def child_count(self):
        """The number of children. Calling this method will trigger loading."""
        return len(self.children_())

    def child_count_loaded(self):
        """The number of children already loaded."""
        return len(self._children) if self._children_update_started else 0

    def isSelectionModified(self):
        return False

    def sort(self, column, order):
        pass


class MessageTreeItem(AbstractTreeItem):
    """A tree item to display a message instead of contents."""

    def __init__(self, parent=None, message=""):
        AbstractTreeItem.__init__(self, parent=parent)
        self._parent = parent
        self._message = message
        self._checkState = None
        self.can_have_children = False

    def _async_loading_done(self, result):
        pass

    def _create_children_async(self):
        pass

    def child_at(self, row):
        return None

    def data(self, column):
        return (self._message, "")[column]

    def header(self):
        return ["Name", "Included"]


def _sort_key(item, column, reverse):
    if column == 0 and isinstance(item, DropboxPathItem):
        if item.is_folder is not reverse:
            prefix = "\x00"
        else:
            prefix = ""

        return f"{prefix}{item._basename.lower()}"

    elif column == 1:
        return item.checkState
    else:
        return item.data(column)


class DropboxPathItem(AbstractTreeItem):
    """A Dropbox folder item. It lists its children asynchronously, only when asked to
    by `TreeModel`."""

    def __init__(
        self,
        async_loader,
        unchecked,
        path_display="/",
        path_lower="/",
        is_folder=True,
        parent=None,
    ):
        super().__init__(parent=parent)
        if is_folder:
            self.icon = native_folder_icon()
            self._children = [MessageTreeItem(self, "Loading...")]
        else:
            self.icon = native_file_icon()
            self._children = []
        self.is_folder = is_folder
        self.can_have_children = is_folder
        self._path_display = path_display
        self._path_lower = path_lower
        self._basename = os.path.basename(self._path_display)
        self._async_loader = async_loader
        self._unchecked = unchecked

        self._checkStateChanged = False

        # get info from our own excluded list
        if path_lower in unchecked:
            # item is excluded
            self._originalCheckState = 0
        elif self._parent is not None and self._parent._originalCheckState == 0:
            # item's parent is excluded
            self._originalCheckState = 0
        elif any(is_child(f, path_lower) for f in unchecked):
            # some of item's children are excluded
            self._originalCheckState = 1
        else:
            # item is fully included
            self._originalCheckState = 2

        # overwrite original state if the parent was modified
        if (
            self._parent is not None
            and self._parent._checkStateChanged
            and not self._parent.checkState == 1
        ):
            # inherit from parent
            self._checkState = self._parent.checkState
            self._checkStateChanged = self._parent._checkStateChanged
        else:
            self._checkStateChanged = False
            self._checkState = int(self._originalCheckState)

    def _create_children_async(self):
        if self.is_folder:
            self._remote = self._async_loader.listChildren(self._path_lower)
            self._remote.sig_result.connect(self._async_loading_done)
        else:
            self._async_loading_done([])

    def _async_loading_done(self, results):
        if isinstance(results, Exception):
            raise results

        for child in self._children.copy():
            if isinstance(child, MessageTreeItem):
                self._children.remove(child)

        if results is False:
            self.loading_failed.emit()
        else:
            new_nodes = [
                DropboxPathItem(
                    self._async_loader,
                    self._unchecked,
                    path_display=e.path_display,
                    path_lower=e.path_lower,
                    is_folder=isinstance(e, FolderMetadata),
                    parent=self,
                )
                for e in results
            ]
            self._children.extend(new_nodes)
            self.sort(0, Qt.SortOrder.AscendingOrder)
            self.loading_done.emit()

    def data(self, column):
        return (self._basename, "")[column]

    def header(self):
        return ["Name", "Included"]

    def _checkStatePropagateToChildren(self, state):
        # propagate to children if checked or unchecked
        if state in (0, 2) and self.child_count_loaded() > 0:
            for child in self._children:
                child._checkStateChanged = True
                child._checkState = state
                child._checkStatePropagateToChildren(state)

    def _checkStatePropagateToParent(self, state):
        # propagate to parent if checked or unchecked
        if self._parent:
            self._parent._checkStateChanged = True
            # get minimum of all other children's check state
            checkstate_other_children = min(
                c.checkState for c in self._parent._children
            )
            # set parent's state to that minimum, if it >= 1 (there always could be
            # included files)
            new_parent_state = max([checkstate_other_children, 1])
            self._parent._checkState = new_parent_state
            # tell the parent to propagate its own state upwards
            self._parent._checkStatePropagateToParent(state)

    @property
    def checkStateChanged(self):
        return self._checkStateChanged

    def isSelectionModified(self):
        own_selection_modified = self._checkState != self._originalCheckState
        child_selection_modified = any(c.isSelectionModified() for c in self._children)
        return own_selection_modified or child_selection_modified

    def sort(self, column, order):
        reverse = order == Qt.SortOrder.DescendingOrder

        self._children.sort(
            key=lambda x: _sort_key(x, column, reverse), reverse=reverse
        )

        for child in self._children:
            child.sort(column, order)


class AsyncListFolder(QtCore.QObject):
    def __init__(self, config_name, parent=None):
        """
        A helper which creates instances of :class:`BackgroundTask` to
        asynchronously list Dropbox folders

        :param str config_name: Config name of Maestral instance
        :param parent: QObject. Defaults to None.
        """
        super().__init__(parent=parent)
        self.config_name = config_name
        self._abort_event = threading.Event()

    def abortListing(self):
        self._abort_event.set()

    def listChildren(self, path):
        """
        Returns a running instance of :class:`maestral.gui.utils.BackgroundTask` which
        will emit `sig_result` once it has a result.
        :param str path: Dropbox path to list.
        :returns: Running background task.
        :rtype: :class:`maestral.gui.utils.BackgroundTask`
        """

        new_job = BackgroundTask(parent=self, target=self._listChildren, args=(path,))

        return new_job

    def _listChildren(self, path):
        """The actual function which does the listing. Returns an iterator over the
        entries in the Dropbox folder."""

        # use a duplicate proxy to prevent blocking of the main connection
        with MaestralProxy(self.config_name) as m:
            entries_iterator = m.list_folder_iterator(path)

            while not self._abort_event.is_set():
                try:
                    entries = next(entries_iterator)
                    entries.sort(key=lambda e: e.name.lower())
                except (NotAFolderError, NotFoundError):
                    entries = []
                except ConnectionError:
                    entries = False
                except StopIteration:
                    return

                yield entries


# noinspection PyArgumentList
class SelectiveSyncDialog(QtWidgets.QDialog, Ui_SelectiveSyncDialog):
    def __init__(self, mdbx, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.setModal(True)

        self.mdbx = mdbx
        self.dbx_model = None
        self.updateButton.setEnabled(False)

        self.ui_failed()

        # connect callbacks
        self.updateButton.clicked.connect(self.on_accepted)
        self.cancelButton.clicked.connect(self.close)
        self.selectAllCheckBox.clicked.connect(self.on_select_all_clicked)

    def populate_folders_list(self, overload=None):
        self.excluded_items = set(self.mdbx.excluded_items)
        self.async_loader = AsyncListFolder(self.mdbx.config_name, self)
        self.dbx_root = DropboxPathItem(
            async_loader=self.async_loader,
            unchecked=set(self.mdbx.excluded_items),
        )
        self.dbx_model = FileSystemModel(self.dbx_root)
        self.dbx_model.loading_done.connect(self.ui_loaded)
        self.dbx_model.loading_failed.connect(self.ui_failed)
        self.dbx_model.dataChanged.connect(self.update_select_all_checkbox)
        self.dbx_model.dataChanged.connect(self.update_dialog_buttons)
        self.treeViewFolders.setModel(self.dbx_model)

    def update_select_all_checkbox(self):
        check_states = []
        for irow in range(self.dbx_model._root_item.child_count_loaded()):
            index = self.dbx_model.index(irow, 1, QModelIndex())
            check_states.append(
                self.dbx_model.data(index, Qt.ItemDataRole.CheckStateRole)
            )

        if all(cs == 2 for cs in check_states):
            self.selectAllCheckBox.setChecked(True)
        else:
            self.selectAllCheckBox.setChecked(False)

    def update_dialog_buttons(self):
        self.updateButton.setEnabled(self.dbx_root.isSelectionModified())

    def on_select_all_clicked(self, checked):
        checked_state = 2 if checked else 0
        for irow in range(self.dbx_model._root_item.child_count_loaded()):
            index = self.dbx_model.index(irow, 1, QModelIndex())
            self.dbx_model.setCheckState(index, checked_state)

    def closeEvent(self, event):
        super().closeEvent(event)
        self.async_loader.abortListing()

    def on_accepted(self, overload=None):
        """
        Apply changes to local Dropbox folder.
        """
        if not self.mdbx.connected:
            self.dbx_model.on_loading_failed()

        else:
            excluded_items = self.get_excluded_items()
            try:
                self.mdbx.excluded_items = excluded_items
            except BusyError as err:
                msg_box = UserDialog(err.title, err.message, parent=self)
                msg_box.open()
            else:
                self.accept()

    def get_excluded_items(self):
        # We load the old excluded list first. This is to preserve any exclusions of
        # sub-folders which may not be loaded in the GUI. We then update the excluded
        # list to reflect any changes made by the user in the GUI.
        excluded_items = set(self.mdbx.excluded_items)

        queue = Queue()
        queue.put(self.dbx_model._root_item)

        while not queue.empty():
            node = queue.get()

            # Include items which have been checked / partially checked.
            # Remove items which have been unchecked.
            # The list will be cleaned up later.

            if node.checkState == 0:
                # Path is fully excluded. Add to excluded list.
                excluded_items.add(node._path_lower)
            elif node.checkState == 1:
                # Path is included but has excluded children. Remove only the path
                # itself from the excluded list.
                excluded_items.discard(node._path_lower)
            elif node.checkState == 2:
                # Path is fully included. Remove it and all its children
                # from excluded list. We do this here because children might not
                # have been loaded / expanded and won't be visited during traversal.
                for path in excluded_items.copy():
                    if is_equal_or_child(path, node._path_lower):
                        excluded_items.discard(path)

            for child in node._children:
                if isinstance(child, DropboxPathItem):
                    queue.put(child)

        return excluded_items

    def ui_failed(self):
        self.updateButton.setEnabled(False)
        self.selectAllCheckBox.setEnabled(False)

    def ui_loaded(self):
        self.selectAllCheckBox.setEnabled(True)
        self.treeViewFolders.resizeColumnToContents(0)

    def changeEvent(self, event):
        if event.type() == QtCore.QEvent.Type.PaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        if self.dbx_model:
            self.dbx_model.reloadData(
                [Qt.ItemDataRole.DecorationRole]
            )  # reload folder icons



# -*- coding: utf-8 -*-

# system imports
import os.path as osp
from urllib import parse

# external packages
import click
from PyQt6 import QtCore, QtGui, QtWidgets
from maestral.utils import sanitize_string

# local imports
from .utils import (
    icon_to_pixmap,
    get_scaled_font,
    center_window,
    is_dark_window,
    LINE_COLOR_DARK,
    LINE_COLOR_LIGHT,
)
from .resources import native_item_icon
from .resources.ui_sync_issues_window import Ui_SyncIssuesWindow
from .resources.ui_sync_issue_widget import Ui_SyncIssueWidget


# noinspection PyArgumentList
class SyncIssueWidget(QtWidgets.QWidget, Ui_SyncIssueWidget):
    """
    A widget to graphically display a Maestral sync issue.
    """

    def __init__(self, sync_err, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.sync_err = sync_err

        dbx_address = "https://www.dropbox.com/preview"
        file_address = parse.quote(self.sync_err.dbx_path)
        self.dropbox_url = dbx_address + file_address

        self.errorLabel.setFont(get_scaled_font(scaling=0.85))
        self.update_dark_mode()  # set appropriate item icon and colors in style sheet

        self.pathLabel.setText(sanitize_string(osp.basename(self.sync_err.dbx_path)))
        self.errorLabel.setText(f"{self.sync_err.title}:\n{self.sync_err.message}")

        def request_context_menu():
            self.actionButton.customContextMenuRequested.emit(self.actionButton.pos())

        self.actionButton.setContextMenuPolicy(
            QtCore.Qt.ContextMenuPolicy.CustomContextMenu
        )
        self.actionButton.pressed.connect(request_context_menu)
        self.actionButton.customContextMenuRequested.connect(self.showContextMenu)

    def showContextMenu(self, pos):
        self.contentMenu = QtWidgets.QMenu()
        a0 = self.contentMenu.addAction("View in folder")
        a1 = self.contentMenu.addAction("View on dropbox.com")

        def launch_local():
            click.launch(self.sync_err.local_path, locate=True)

        def launch_online():
            click.launch(self.dropbox_url)

        a0.triggered.connect(launch_local)
        a1.triggered.connect(launch_online)

        a0.setEnabled(osp.exists(self.sync_err.local_path))
        a1.setEnabled(True)

        self.contentMenu.exec(self.mapToGlobal(pos))

    def changeEvent(self, event):
        if event.type() == QtCore.QEvent.Type.PaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        # update style sheet with new colors
        line_rgb = LINE_COLOR_DARK if is_dark_window() else LINE_COLOR_LIGHT
        bg_color = self.palette().color(QtGui.QPalette.ColorRole.Base)
        bg_color_rgb = [bg_color.red(), bg_color.green(), bg_color.blue()]
        self.frame.setStyleSheet(
            """
        .QFrame {{
            border: 1px solid rgb({0},{1},{2});
            background-color: rgb({3},{4},{5});
            border-radius: 7px;
        }}""".format(
                *line_rgb, *bg_color_rgb
            )
        )

        # update item icons (the system may supply different icons in dark mode)
        icon = native_item_icon(self.sync_err.local_path)
        pixmap = icon_to_pixmap(icon, self.iconLabel.width(), self.iconLabel.height())
        self.iconLabel.setPixmap(pixmap)


# noinspection PyArgumentList
class SyncIssueWindow(QtWidgets.QWidget, Ui_SyncIssuesWindow):
    """
    A widget to graphically display all Maestral sync issues.
    """

    def __init__(self, mdbx, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.mdbx = mdbx
        self.sync_issue_widgets = []

        self.refresh_gui()

        center_window(self)

        self.update_timer = QtCore.QTimer()
        self.update_timer.timeout.connect(self.refresh_gui)
        self.update_timer.start(1000)  # every 1 sec

    def refresh_gui(self):
        sync_errors_list = self.mdbx.sync_errors  # get a new copy

        self.clear()

        if len(sync_errors_list) == 0:
            no_issues_label = QtWidgets.QLabel("No sync issues :)")
            self.verticalLayout.addWidget(no_issues_label)
            self.sync_issue_widgets.append(no_issues_label)

        for issue in sync_errors_list:
            self.add_issue(issue)

    def add_issue(self, sync_issue):
        issue_widget = SyncIssueWidget(sync_issue)
        self.sync_issue_widgets.append(issue_widget)
        self.verticalLayout.addWidget(issue_widget)

    def clear(self):
        while self.verticalLayout.itemAt(0):
            item = self.verticalLayout.takeAt(0)
            w = item.widget()
            if w:
                w.deleteLater()

        self.sync_issue_widgets.clear()

    def show(self):
        self.update_timer.start()
        return super().show()

    def close(self):
        self.update_timer.stop()
        return super().close()



# Form implementation generated from reading ui file 'bandwidth_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.4.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_BandwidthDialog(object):
    def setupUi(self, BandwidthDialog):
        BandwidthDialog.setObjectName("BandwidthDialog")
        BandwidthDialog.resize(356, 255)
        self.gridLayout = QtWidgets.QGridLayout(BandwidthDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.updateButton = QtWidgets.QPushButton(parent=BandwidthDialog)
        self.updateButton.setObjectName("updateButton")
        self.gridLayout.addWidget(self.updateButton, 4, 2, 1, 1)
        self.cancelButton = QtWidgets.QPushButton(parent=BandwidthDialog)
        self.cancelButton.setObjectName("cancelButton")
        self.gridLayout.addWidget(self.cancelButton, 4, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 4, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(parent=BandwidthDialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridlayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridlayout.setObjectName("gridlayout")
        self.radioButtonUploadLimited = QtWidgets.QRadioButton(parent=self.groupBox_2)
        self.radioButtonUploadLimited.setObjectName("radioButtonUploadLimited")
        self.gridlayout.addWidget(self.radioButtonUploadLimited, 1, 0, 1, 1)
        self.numberInputUploadRate = QtWidgets.QDoubleSpinBox(parent=self.groupBox_2)
        self.numberInputUploadRate.setProperty("value", 1.0)
        self.numberInputUploadRate.setObjectName("numberInputUploadRate")
        self.gridlayout.addWidget(self.numberInputUploadRate, 1, 1, 1, 1)
        self.radioButtonUploadUnlimited = QtWidgets.QRadioButton(parent=self.groupBox_2)
        self.radioButtonUploadUnlimited.setObjectName("radioButtonUploadUnlimited")
        self.gridlayout.addWidget(self.radioButtonUploadUnlimited, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label.setObjectName("label")
        self.gridlayout.addWidget(self.label, 1, 2, 1, 1)
        self.gridLayout.addWidget(self.groupBox_2, 2, 0, 1, 3)
        self.groupBox = QtWidgets.QGroupBox(parent=BandwidthDialog)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.radioButtonDownloadUnlimited = QtWidgets.QRadioButton(parent=self.groupBox)
        self.radioButtonDownloadUnlimited.setObjectName("radioButtonDownloadUnlimited")
        self.gridLayout_2.addWidget(self.radioButtonDownloadUnlimited, 1, 1, 1, 1)
        self.numberInputDownloadRate = QtWidgets.QDoubleSpinBox(parent=self.groupBox)
        self.numberInputDownloadRate.setProperty("value", 1.0)
        self.numberInputDownloadRate.setObjectName("numberInputDownloadRate")
        self.gridLayout_2.addWidget(self.numberInputDownloadRate, 2, 2, 1, 1)
        self.radioButtonDownloadLimited = QtWidgets.QRadioButton(parent=self.groupBox)
        self.radioButtonDownloadLimited.setObjectName("radioButtonDownloadLimited")
        self.gridLayout_2.addWidget(self.radioButtonDownloadLimited, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 2, 3, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 2, 3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 3)

        self.retranslateUi(BandwidthDialog)
        QtCore.QMetaObject.connectSlotsByName(BandwidthDialog)

    def retranslateUi(self, BandwidthDialog):
        _translate = QtCore.QCoreApplication.translate
        BandwidthDialog.setWindowTitle(_translate("BandwidthDialog", "Bandwidth Dialog"))
        self.updateButton.setText(_translate("BandwidthDialog", "Update"))
        self.cancelButton.setText(_translate("BandwidthDialog", "Cancel"))
        self.groupBox_2.setTitle(_translate("BandwidthDialog", "Upload rate"))
        self.radioButtonUploadLimited.setText(_translate("BandwidthDialog", "Limit"))
        self.radioButtonUploadUnlimited.setText(_translate("BandwidthDialog", "Don\'t limit"))
        self.label.setText(_translate("BandwidthDialog", "MB/s"))
        self.groupBox.setTitle(_translate("BandwidthDialog", "Download rate"))
        self.radioButtonDownloadUnlimited.setText(_translate("BandwidthDialog", "Don\'t limit"))
        self.radioButtonDownloadLimited.setText(_translate("BandwidthDialog", "Limit"))
        self.label_3.setText(_translate("BandwidthDialog", "MB/s"))



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 31 16:23:13 2018

@author: samschott
"""

# system imports
import os.path as osp

# external imports
from PyQt6 import QtGui, QtCore, QtWidgets
from PyQt6.QtCore import Qt

# maestral modules
from maestral.utils.appdirs import get_home_dir
from maestral.utils.path import delete
from maestral.daemon import stop_maestral_daemon_process

# local imports
from .resources import APP_ICON_PATH, native_folder_icon
from .resources.ui_dropbox_location_dialog import Ui_Dialog
from .utils import MaestralBackgroundTask, icon_to_pixmap, is_empty
from .widgets import UserDialog


# noinspection PyArgumentList
class DropboxLocationDialog(QtWidgets.QDialog, Ui_Dialog):
    """A dialog to link and set up a new Dropbox account."""

    accepted = False

    def __init__(self, mdbx, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        # noinspection PyTypeChecker
        self.setWindowFlags(
            Qt.WindowType.WindowStaysOnTopHint
            | Qt.WindowType.Sheet
            | Qt.WindowType.WindowTitleHint
            | Qt.WindowType.CustomizeWindowHint
        )

        self.mdbx = mdbx
        self.config_name = self.mdbx.config_name

        self.app_icon = QtGui.QIcon(APP_ICON_PATH)

        self.labelIcon.setPixmap(icon_to_pixmap(self.app_icon, 60))

        # set up Dropbox location combobox

        self.dropbox_location = self.mdbx.get_conf("sync", "path")

        if self.dropbox_location == "":
            folder_name = f"Dropbox ({self.config_name.capitalize()})"
            self.dropbox_location = osp.join(get_home_dir(), folder_name)

        self.comboBoxPath.addItem(native_folder_icon(), self.dropbox_location)
        self.comboBoxPath.insertSeparator(1)
        self.comboBoxPath.addItem(QtGui.QIcon(), "Choose...")
        self.comboBoxPath.currentIndexChanged.connect(self.on_combobox)

        self.dropbox_folder_dialog = QtWidgets.QFileDialog(self)
        self.dropbox_folder_dialog.setAcceptMode(
            QtWidgets.QFileDialog.AcceptMode.AcceptOpen
        )
        self.dropbox_folder_dialog.setFileMode(QtWidgets.QFileDialog.FileMode.Directory)
        self.dropbox_folder_dialog.setOption(
            QtWidgets.QFileDialog.Option.ShowDirsOnly, True
        )
        self.dropbox_folder_dialog.setLabelText(
            QtWidgets.QFileDialog.DialogLabel.Accept, "Select"
        )
        self.dropbox_folder_dialog.setDirectory(get_home_dir())
        self.dropbox_folder_dialog.fileSelected.connect(self.on_new_dbx_folder)
        self.dropbox_folder_dialog.rejected.connect(
            lambda: self.comboBoxPath.setCurrentIndex(0)
        )

        self.pushButtonSelect.setDefault(True)

        # connect buttons to callbacks
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.pushButtonQuit.clicked.connect(self.on_quit_clicked)
        self.pushButtonSelect.clicked.connect(self.on_selected_clicked)
        self.pushButtonUnlink.clicked.connect(self.on_unlink_clicked)

    # =============================================================================
    # Main callbacks
    # =============================================================================

    @QtCore.pyqtSlot()
    def on_quit_clicked(self):
        self._deactivate_buttons()
        self.spinner.startAnimation()
        stop_maestral_daemon_process(self.mdbx.config_name)

    @QtCore.pyqtSlot()
    def on_unlink_clicked(self):
        self._deactivate_buttons()
        self.spinner.startAnimation()

        self.unlink_task = MaestralBackgroundTask(
            parent=self,
            config_name=self.mdbx.config_name,
            target="unlink",
        )

        self.unlink_task.sig_result.connect(self.on_unlink_done)

    @QtCore.pyqtSlot()
    def on_unlink_done(self):
        self.spinner.stopAnimation()
        stop_maestral_daemon_process(self.mdbx.config_name)

    @QtCore.pyqtSlot()
    def on_selected_clicked(self):
        # apply dropbox path
        try:
            if osp.exists(self.dropbox_location):
                if is_empty(self.dropbox_location):
                    delete(self.dropbox_location, raise_error=True)
                else:
                    msg_box = UserDialog(
                        title="Folder is not empty",
                        message=(
                            f'The folder "{osp.basename(self.dropbox_location)}" is '
                            "not empty. Would you like to merge its content with your "
                            "Dropbox?"
                        ),
                        button_names=("Cancel", "Merge"),
                        parent=self,
                    )
                    res = msg_box.exec()

                    if res == UserDialog.DialogCode.Accepted:
                        return
                    elif res == UserDialog.DialogCode.Rejected:
                        pass

            self.mdbx.create_dropbox_directory(self.dropbox_location)
        except OSError:
            msg_box = UserDialog(
                title="Could not set directory",
                message=(
                    "Please check if you have permissions to write to the "
                    "selected location."
                ),
                parent=self,
            )
            msg_box.exec()
            return

        # Resume sync with clean sync state.
        self.mdbx.reset_sync_state()
        self.mdbx.start_sync()
        self.close()

    # =============================================================================
    # Helper functions
    # =============================================================================

    def _deactivate_buttons(self):
        self.pushButtonQuit.setEnabled(False)
        self.pushButtonSelect.setEnabled(False)
        self.pushButtonUnlink.setEnabled(False)
        self.comboBoxPath.setEnabled(False)

    @QtCore.pyqtSlot(int)
    def on_combobox(self, idx):
        if idx == 2:
            self.dropbox_folder_dialog.open()

    @QtCore.pyqtSlot(str)
    def on_new_dbx_folder(self, new_location):
        self.comboBoxPath.setCurrentIndex(0)
        if not new_location == "":
            self.comboBoxPath.setItemText(0, new_location)

        self.dropbox_location = new_location

    def changeEvent(self, event):
        if event.type() == QtCore.QEvent.Type.PaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        if self.dbx_model:
            # reload folder icons
            self.dbx_model.reloadData([Qt.ItemDataRole.DecorationRole])



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 31 16:23:13 2018

@author: samschott
"""
import os

__author__ = "Sam Schott"
__version__ = "1.7.2.dev0"
__url__ = "https://maestral.app"

# add '~/.local/share' to XDG_DATA_DIRS
# this is needed to find icons installed for the current user only
XDG_DATA_DIRS = os.environ.get("XDG_DATA_DIRS", "")
XDG_DATA_DIRS += ":" + os.path.expanduser("~/.local/share")
os.environ["XDG_DATA_DIRS"] = XDG_DATA_DIRS.strip(":")
